package element;

/**
 * Elemento compuesto que tiene dentro un nmero finito y conocido
 * de elementos. Estos composites se utilizarn slo en el rbol de parsing que representa un "parsing schema", y no en el
 * cdigo generado, donde podemos aumentar la eficiencia utilizando arrays.
 * @author carlos
 */
public abstract class CompositeElement implements Element , ElementAggregate
{

    /**
     * Soporte del patrn de diseo Visitante para los distintos tipos de elemento.
     */
	public void accept ( ElementVisitor v )
	{
		v.visitCompositeElement(this);
	}
	
	/**
	 * Obtiene los elementos hoja de este elemento compuesto, es decir, aquellos
	 * elementos pertenecientes a la clase SimpleElement que estn dentro de este
	 * compuesto.
	 * @return Elementos hoja del rbol de elementos que parte de este composite.
	 */
	public java.util.List getLeafElements()
	{
	    java.util.List result = new java.util.ArrayList();
	    for ( int i = 0 ; i < numChildren() ; i++ )
	    {
	        Element child = getChild(i);
	        if ( child instanceof CompositeElement )
	        {
	            //elemento compuesto: aadir hojas a la lista (llamada recursiva).
	            CompositeElement compChild = (CompositeElement) child;
	            result.addAll(compChild.getLeafElements());
	        }
	        else
	        {
	            //elemento hoja: aadirlo a la lista.
	            result.add(child);
	        }
	    }
	    return result;
	}
	
	/**
	 * Comparacin de igualdad para elementos compuestos. Dos elementos compuestos
	 * sern iguales si tienen el mismo nmero de hijos y sus hijos son iguales.
	 */
	public boolean equals ( Object o )
	{
		if ( !(o instanceof CompositeElement) )
			return false;
		else
		{
			int nchildren = ((CompositeElement)o).numChildren();
			for ( int i = 0 ; i < nchildren ; i++ )
			{
				if ( ! this.getChild(i).equals(((CompositeElement)o).getChild(i) ) )
					return false;
			}
			return true;
		}
	}
	
}
