/*
 * Created on 12-ago-2004 13:17:47
 *
 */
package element;

/**
 * This kind of elements represent either a string position of a value called
 * "undefined".
 * @author Carlos
 *
 */
public class NullableStringPosition extends StringPosition
{
    
	private boolean isUndefined;
	
	public boolean isUndefined()
	{
		return isUndefined;
	}
	
    /**
     * @param value Value from the string position.
     * @param ground true if the string position is ground.
     * @param isUndefined true if the string position is undefined.
     */
    public NullableStringPosition ( boolean isUndefined , int value, boolean ground )
    {
        super ( value, ground );
        this.isUndefined = isUndefined;
    }
    
    public static int getArrayLength()
    {
        return getMaxValue() + 1;
    }
    
    /**
     * Mximo valor que puede tomar una posicin de cadena de entrada. Depender de la
     * frase, y nos da su longitud (n).
     * @return Longitud de la frase, o mximo valor de una posicin de cadena
     * de entrada.
     */
    public static int getMaxValue()
    {
    	return StringPosition.getMaxValue();
    }
    
    private static int valuesForNonGround = 0;
    private static java.util.HashMap nonGroundHash = new java.util.HashMap(); //keys: String
    private static java.util.HashMap groundHash = new java.util.HashMap(); //keys: Integer
    
	/**
	 * Mtodo de parsing para StringPositionOrNulls, nos da la posicin asociada a una cadena dada.
	 * (non ground)
	 * @return Smbolo correspondiente a la cadena dada.
	 */
	public static NullableStringPosition nonGroundNullableStringPosition(String s)
	{
	    if ( nonGroundHash.get(s) != null )
	        return (NullableStringPosition) nonGroundHash.get(s);
	    else
	    {
	        //nueva
	        valuesForNonGround++;
	        NullableStringPosition nueva = new NullableStringPosition(false,valuesForNonGround,false);
	        //notese que al ser no ground, el primer false realmente no hace nada.
	        nonGroundHash.put(s,nueva);
	        return nueva;
	    }
	}
	
	/**
	 * This is for using at generated expression code.
	 */
	public static NullableStringPosition groundFromValue ( boolean undef , int val )
	{
		if ( undef )
			return groundNullFromString("-");
		else
			return groundNonNullFromValue(val);
	}
	
	public static NullableStringPosition groundNonNullFromValue(Integer i)
	{
		NullableStringPosition sp = (NullableStringPosition) groundHash.get(i);
	    if ( sp != null )
	    {
	        return sp;
	    }
	    else
	    {
	        //nueva
	        NullableStringPosition nueva = new NullableStringPosition(false,i.intValue(),true);
	        groundHash.put(i,nueva);
	        return nueva;
	    }
	}
	
	private static NullableStringPosition groundNull = new NullableStringPosition(true,0,true);
	
	public static NullableStringPosition groundNullFromString(String s)
	{
		return groundNull;
	}
		
	
	public static NullableStringPosition groundNonNullFromValue ( int i )
	{
		if ( i<= getMaxValue() && getMaxValue() != 0 )
			return groundNonNullFromValue ( commonIntegerInstances[i] );
		else
			return groundNonNullFromValue ( new Integer(i) );
	}
	
	public static StringPosition groundNonNullFromString(String sval)
	{
	    return groundNonNullFromValue(Integer.valueOf(sval));
	}
	
	/**
	 * En el caso de las StringPosition ground, devolvemos una cadena con su valor
	 * (p.ej. "3"), y en el caso de las no-ground, devolvemos un entero prefijado con
	 * una i (i1, i2, etc.)
	 */
	public String toString()
	{
	    if ( isGround() )
	    {
	    	if ( isUndefined )
	    		return "-";
	    	else
	    		return ""+value;
	    }
	    else
	        return "p"+value;
	}
	
	/**
	 * Devuelve el cdigo Java para que las posiciones ground aparezcan en expresiones.
	 */
	public String getExpressionCode()
	{
	    if ( ground )
	        return "NullableStringPosition.groundFromValue("+isUndefined+","+value+")";
	    else
	        return null;
	}
	
	/**
	 * En este caso, al contrario que en los IntElements generales, equals hace
	 * por dentro una comparacin de identidad.
	 */
	public boolean equals ( Object o )
	{
		return ( o != null && this == o );
	}
	
	/**
	 * Implements the union operation for nullable string positions.
	 */
	public NullableStringPosition union ( NullableStringPosition another )
	{
		if ( !this.isUndefined )
			return this;
		else
			return another;
	}
	
	
	
}
