package element.predicate;

import java.util.List;

import parser.eparser.ElementParseException;
import parser.eparser.ElementParser;
import element.BooleanElement;
import element.Element;
import element.ExpressionElement;
import element.SumOfIntegersExpression;

/**
 * Esta clase representa una expresin booleana de tipo Identity(A,B), que se
 * traduce a una comparacin de identidad (==) de Java.
 * @author Carlos
 *
 */
public class IdentityExpression extends ExpressionElement 
{

	public IdentityExpression(Element argument1 , Element argument2)
	{
		super( new Element[] { argument1 , argument2 } );
	}
	
	public boolean isGround()
	{
		return false;
	}
	
	public Class getResultingClass()
	{
		return BooleanElement.class;
	}
	
	 /**
     * La expresin IdentityExpression se resuelve a un cdigo Java que llama
     * a la comparacin de identidad ==.
     */
	public String getExpressionCode(String[] memberNames) 
	{
        StringBuffer result = new StringBuffer("");
        
        result.append("BooleanElement.groundFromValue ( ");
        
        result.append("(");
        result.append(memberNames[0]);
        result.append(")");
        result.append( " == " );
        result.append("(");
        result.append(memberNames[1]);
        result.append(")");
        
        result.append(" )");
        
        return result.toString();
	}
	
	public static IdentityExpression fromString ( String sparm ) throws ElementParseException
	{

		try
		{
			FunctionExpressionParser fep = new FunctionExpressionParser(sparm);
			
			List expressionArgs = fep.getArguments();
			
			if ( expressionArgs.size() != 2 )
			{
				throw new ElementParseException("Identity expression taking " + expressionArgs.size() + " arguments when 2 are expected");
			}
			
			Element argument1 = ElementParser.getCurrentInstance().parse(((String)expressionArgs.get(0)));
			Element argument2 = ElementParser.getCurrentInstance().parse(((String)expressionArgs.get(1)));
			
			return new IdentityExpression ( argument1 , argument2 );
			
		}
		catch ( FunctionExpressionParseException fepe )
		{
			throw new ElementParseException(fepe);
		}
		
	}
	
}
