package element.tag.predicate;

import java.util.List;

import parser.eparser.ElementParseException;
import parser.eparser.ElementParser;
import element.BooleanElement;
import element.Element;
import element.ExpressionElement;
import element.SumOfIntegersExpression;
import element.predicate.FunctionExpressionParseException;
import element.predicate.FunctionExpressionParser;

/**
 * Represents an expression of the form Anchor-of(tree) that
 * obtains a tree's anchor node.
 */
public class AnchorOfExpression extends ExpressionElement 
{

	public AnchorOfExpression(Element argument1 )
	{
		super( new Element[] { argument1 } );
	}
	
	public boolean isGround()
	{
		return false;
	}
	
	public Class getResultingClass()
	{
		return element.tag.TAGTreeNode.class;
	}
	
	public String getExpressionCode(String[] memberNames) 
	{
        StringBuffer result = new StringBuffer("(");
        result.append(memberNames[0]);
        result.append(".getAnchor()");
        result.append(")");
        return result.toString();
	}
	
	public static AnchorOfExpression fromString ( String sparm ) throws ElementParseException
	{

		try
		{
			FunctionExpressionParser fep = new FunctionExpressionParser(sparm);
			
			List expressionArgs = fep.getArguments();
			
			if ( expressionArgs.size() != 1 )
			{
				throw new ElementParseException("Anchor Of expression taking " + expressionArgs.size() + " arguments when 1 is expected");
			}
			
			Element argument1 = ElementParser.getCurrentInstance().parse(((String)expressionArgs.get(0)));
			
			return new AnchorOfExpression ( argument1 );
			
		}
		catch ( FunctionExpressionParseException fepe )
		{
			throw new ElementParseException(fepe);
		}
		
	}
	
}
