package element.tag.predicate;

import java.util.List;

import parser.eparser.ElementParseException;
import parser.eparser.ElementParser;
import element.BooleanElement;
import element.Element;
import element.ExpressionElement;
import element.SumOfIntegersExpression;
import element.predicate.FunctionExpressionParseException;
import element.predicate.FunctionExpressionParser;

/**
 * Represents a boolean expression of the form Has-valid-anchor-for ( tree ; symbol ; fs )
 * @author Carlos
 *
 */
public class HasValidAnchorForExpression extends ExpressionElement 
{

	public HasValidAnchorForExpression(Element argument1 , Element argument2 , Element argument3)
	{
		super( new Element[] { argument1 , argument2 , argument3 } );
	}
	
	public boolean isGround()
	{
		return false;
	}
	
	public Class getResultingClass()
	{
		return BooleanElement.class;
	}
	
	public String getExpressionCode(String[] memberNames) 
	{
        StringBuffer result = new StringBuffer("");
        result.append("BooleanElement.groundFromValue(");
        result.append(memberNames[0]);
        result.append(".hasValidAnchorFor( ");
        result.append(memberNames[1]);
        result.append(",");
        result.append(memberNames[2]);
        result.append(" )");
        result.append(")");
        return result.toString();
	}
	
	public static HasValidAnchorForExpression fromString ( String sparm ) throws ElementParseException
	{

		try
		{
			FunctionExpressionParser fep = new FunctionExpressionParser(sparm);
			
			List expressionArgs = fep.getArguments();
			
			if ( expressionArgs.size() != 3 )
			{
				throw new ElementParseException("Has Valid Anchor For expression taking " + expressionArgs.size() + " arguments when 3 are expected");
			}
			
			Element argument1 = ElementParser.getCurrentInstance().parse(((String)expressionArgs.get(0)));
			Element argument2 = ElementParser.getCurrentInstance().parse(((String)expressionArgs.get(1)));
			Element argument3 = ElementParser.getCurrentInstance().parse(((String)expressionArgs.get(2)));
			
			return new HasValidAnchorForExpression ( argument1 , argument2 , argument3 );
			
		}
		catch ( FunctionExpressionParseException fepe )
		{
			throw new ElementParseException(fepe);
		}
		
	}
	
}
