/*
 * Created at regulus on 11-dic-2005 12:14:11
 * as file TreeAdjoiningGrammarWithFeatureStructures.java on package grammar.tag
 */
package grammar.tag;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.TreeSet;

import util.DOMNodeListWrapper;
import util.DOMNodeWrapper;
import element.fs.PlainFeatureStructure;
import element.tag.TAGTree;
import element.tag.TAGTreeNode;

/**
 * @author carlos
 *
 * Created at regulus, 11-dic-2005 12:14:11
 */
public class TreeAdjoiningGrammarWithFeatureStructures extends TreeAdjoiningGrammar
{

	private Map nodesToTopFeatures = new HashMap();
	private Map nodesToBottomFeatures = new HashMap();
	
	private Map treesToVars = new HashMap(); //list variables from a tree
	
	private void addVarToTree ( TAGTree tree , String varName )
	{
		PlainFeatureStructure pfs = (PlainFeatureStructure) treesToVars.get(tree);
		if ( pfs == null )
		{
			pfs = new PlainFeatureStructure(true);
			treesToVars.put(tree,pfs);
		}
		//pfs.linkFeatureToVariable("var_"+varName,varName);
		pfs.addTreeVariable(tree,varName);
	}
	
	public PlainFeatureStructure getTreeVariablesStructure ( TAGTree tree )
	{
		PlainFeatureStructure pfs = (PlainFeatureStructure) treesToVars.get(tree);
		if ( pfs == null )
			return PlainFeatureStructure.getEmptyInstance();
		else return pfs;
	}
	
	public PlainFeatureStructure getTopFeatureStructure ( TAGTreeNode node )
	{
		PlainFeatureStructure pfs = (PlainFeatureStructure) nodesToTopFeatures.get(node);
		if ( pfs == null )
			return PlainFeatureStructure.getEmptyInstance();
		else return pfs;
		//return (PlainFeatureStructure) nodesToTopFeatures.get(node);
	}
	
	public PlainFeatureStructure getBottomFeatureStructure ( TAGTreeNode node )
	{
		PlainFeatureStructure pfs = (PlainFeatureStructure) nodesToBottomFeatures.get(node);
		if ( pfs == null )
			return PlainFeatureStructure.getEmptyInstance();
		else return pfs;
		//return (PlainFeatureStructure) nodesToBottomFeatures.get(node);
	}
	
	public PlainFeatureStructure initFeatureStructureFromNode ( TAGTreeNode ttn ,DOMNodeWrapper fsNode )
	{
		PlainFeatureStructure result = new PlainFeatureStructure(true);
		DOMNodeListWrapper fNodes = fsNode.getDirectChildrenElementsByTagName("f");
		for ( int i = 0 ; i < fNodes.size() ; i++ )
		{
			//get the ith feature node
			DOMNodeWrapper featNode = fNodes.get(i);
			
			//get the feature name (must exist)
			String featName = featNode.getAttribute("name");
			if ( featName == null )
			{
				System.err.println("[WARNING] unnamed feature");
			}
			
			//get the link (var) name (if it exists)
			String varName = null;
			DOMNodeListWrapper linkNodes = featNode.getDirectChildrenElementsByTagName("link");
			if ( linkNodes.size() > 0 )
			{
				DOMNodeWrapper linkNode = linkNodes.get(0);
				varName = linkNode.getAttribute("id");
			}
			
			//get the value (if it exists)
			String value = null;
			//check if value is present inside a val node
			DOMNodeListWrapper valNodes = featNode.getDirectChildrenElementsByTagName("val");
			if ( valNodes.size() > 0 )
			{
				DOMNodeWrapper valNode = valNodes.get(0);
				DOMNodeListWrapper children = valNode.getChildren();
				if ( children.size() > 0 )
				{
					DOMNodeWrapper child = children.get(0);
					value = child.getNode().getNodeValue();
				}
			}
			//check if value is present as a plus or minus node
			DOMNodeListWrapper plusNodes = featNode.getDirectChildrenElementsByTagName("plus");
			if ( plusNodes.size() > 0 ) value = "+";
			DOMNodeListWrapper minusNodes = featNode.getDirectChildrenElementsByTagName("minus");
			if ( plusNodes.size() > 0 ) value = "-";
			
			//add the gathered data to the feature structure
			if ( varName != null )
			{
				//result.linkFeatureToVariable(featName,varName+"@"+ttn.getAssociatedTree().getName());
				result.linkFeatureToTreeVariable(featName,ttn.getAssociatedTree(),varName);
				//addVarToTree(ttn.getAssociatedTree(),varName+"@"+ttn.getAssociatedTree().getName());
				addVarToTree(ttn.getAssociatedTree(),varName);
			}
			if ( value != null )
			{
				//if value is a disjunction, alphabetically order terms
				if ( value.indexOf("/") >= 0 )
				{
					Set terms = new TreeSet();
					StringTokenizer st = new StringTokenizer(value,"/");
					while ( st.hasMoreTokens() )
						terms.add(st.nextToken());
					value = "";
					Iterator it = terms.iterator();
					value += it.next();
					while ( it.hasNext() )
					{
						value += "/";
						value += it.next();
					}
				}	
				result.setFeatureValue(featName,value);
			}
			
		}
		
		return result;
		
	}
	
	public void initFeatures ( TAGTreeNode ttn , DOMNodeWrapper domn )
	{
		DOMNodeListWrapper nargNodes = 
			domn.getDirectChildrenElementsByTagName("narg");
		for ( int i = 0 ; i < nargNodes.size() ; i++ )
		{
			
			//get the ith narg node
			DOMNodeWrapper nargNode = (DOMNodeWrapper) nargNodes.get(i);
			
			//determine whether it's top or bottom
			String type = nargNode.getAttribute("type");
			boolean top;
			if ( "top".equals(type) )
				top = true;
			else if ( "bot".equals(type) )
				top = false;
			else
			{
				System.err.println("[WARNING] neither top nor bot found at narg node type, type reported as " + type);
				continue;
			}
			
			//get fs (feature structure) node
			DOMNodeListWrapper fsNodes =
				nargNode.getDirectChildrenElementsByTagName("fs");
			if ( fsNodes.size() > 1 )
			{
				System.err.println("[WARNING] don't know how to deal with more than one fs node inside a single narg node");
				continue;
			}
			if ( fsNodes.size() < 1 )
			{
				System.err.println("[WARNING] don't know how to deal with a narg node without an fs node inside");
				continue;
			}
			DOMNodeWrapper fsNode = fsNodes.get(0);
			
			//init feature structure
			PlainFeatureStructure pfs = initFeatureStructureFromNode(ttn,fsNode);
			
			//add feature structure to maps
			if ( top )
			{
				nodesToTopFeatures.put(ttn,pfs);
			}
			else
			{
				nodesToBottomFeatures.put(ttn,pfs);
			}
			
		}
	}
	
}
