package util;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Esta clase es un wrapper sobre un nodo DOM que proporciona una interfaz ms sencilla
 * para trabajar con l.
 * @author Carlos
 *
 */
public class DOMNodeWrapper 
{

	private Node n;
	
	public DOMNodeWrapper ( Node n )
	{
		this.n = n;
	}
	
	public Node getNode()
	{
		return n;
	}
	
	public DOMNodeWrapper getChild(int searchIndex) 
	{
        org.w3c.dom.Node node = n.getChildNodes().item(searchIndex);
        return new DOMNodeWrapper(node); 
	}
	
	public boolean isElement()
	{
		return ( n instanceof Element );
	}
	
	public DOMNodeListWrapper getElementsByTagName( String name )
	{
		if ( isElement() )
			return new DOMNodeListWrapper( ((Element)n).getElementsByTagName(name) );
		else
			return null;
	}
	
	public DOMNodeListWrapper getChildren()
	{
		return new DOMNodeListWrapper ( n.getChildNodes() );
	}
	
	public String getTagName()
	{
		if ( isElement() )
		{
			return ((Element)n).getNodeName();
		}
		else
			return null;
	}
	
	public DOMNodeListWrapper getDirectChildrenElementsByTagName( String name )
	{
		DOMNodeListWrapper children = getChildren();
		final List l = new ArrayList();
		for ( int i = 0 ; i < children.size() ; i++ )
		{
			DOMNodeWrapper node = children.get(i);
			if ( node.isElement() && node.getTagName().equals(name) )
				l.add(node.getNode());
		}
		return new DOMNodeListWrapper ( new NodeList() {

			public Node item(int i) 
			{
				return (Node) l.get(i);
			}

			public int getLength() 
			{
				return l.size();
			}} );
	}
	
	public String getAttribute( String name )
	{
		if ( isElement() )
			return ((Element)n).getAttribute(name);
		else
			return null;
	}
	
}
