/*
 * Created on 12-ago-2004 13:17:47
 *
 */
package element;

/**
 * <p>Elemento que representa un valor booleano, pudiendo tomar los valores
 * false o true.</p>
 * <p>Estando este tipo de elementos acotado, BooleanElement implementa
 * tanto HashIndexable como ArrayIndexable.</p>
 * <p>Created 1-oct-2005 13:05:47</p>
 * @author carlos
 */
public class BooleanElement extends SimpleElement implements HashIndexable, ArrayIndexable
{
    
	private boolean value;
	private boolean ground;
	
    /**
     * @param value Valor que tomar este elemento booleano.
     * @param ground Si el valor es ground o no.
     */
    public BooleanElement ( boolean value, boolean ground )
    {
        this.value = value;
        this.ground = ground;
    }
    
	/**
	 * Nos dice si este booleano es ground (constante) o no.
	 */
	public boolean isGround()
	{
		return ground;
	}

    /**
     * @see element.ArrayIndexable#getArrayIndex()
     */
    public int getArrayIndex ()
    {
        if ( value ) return 1;
        else return 0;
    }
    
	/**
	 * Devuelve el propio objeto BooleanElement.
	 * @see element.HashIndexable#getHashKey()
	 */
	public Object getHashKey() 
	{
		return this;
	}
	    
    /**
     * @see element.ArrayIndexable
     * @return Longitud del array para indexar los elementos booleanos.
     */
    public static int getArrayLength()
    {
        return 2;
    }
    
    /**
     * Devuelve el valor booleano de este elemento.
     */
    public boolean getValue()
    {
    	return value;
    }
        
    private static java.util.HashMap nonGroundHash = new java.util.HashMap(); //keys: String
    
	/**
	 * Mtodo de parsing para BooleanElements, nos da el booleano no ground
	 * asociado a una cadena (nombre simblico) dada.
	 * @return BooleanElement no ground correspondiente a la cadena dada.
	 */
	public static BooleanElement nonGroundFromString(String s)
	{
	    if ( nonGroundHash.get(s) != null )
	        return (BooleanElement) nonGroundHash.get(s);
	    else
	    {
	        //nueva
	        BooleanElement nuevo = new BooleanElement(false,false);
	        nonGroundHash.put(s,nuevo);
	        return nuevo;
	    }
	}
	
	private static BooleanElement GROUND_FALSE = new BooleanElement(false,true);
	private static BooleanElement GROUND_TRUE = new BooleanElement(true,true);
	
	/**
	 * Obtiene un objeto BooleanElement constante (ground) con el valor dado.
	 * Ntese que estos objetos se comparten, es decir, si se obtiene la constante
	 * para el mismo valor varias veces se devolver el mismo objeto.
	 * @param b Valor booleano.
	 * @return Objeto BooleanElement canonicalizado que representa ese valor.
	 */
	public static BooleanElement groundFromValue(Boolean b)
	{
		if ( b.booleanValue() ) return GROUND_TRUE;
		else return GROUND_FALSE;
	}
	
	/**
	 * Lo mismo que groundFromValue(Boolean) pero tomando un boolean bsico
	 * sin wrapper.
	 * @param b Valor booleano.
	 * @return Objeto BooleanElement canonicalizado que representa ese valor.
	 */
	public static BooleanElement groundFromValue(boolean b)
	{
		if ( b ) return GROUND_TRUE;
		else return GROUND_FALSE;
	}
				
	/**
	 * Obtiene un objeto booleano ground (constante) a partir de una cadena
	 * que represente su valor. La cadena tiene que ser "true" o "false".
	 * @param sval Cadena que representa un valor constante asociable a un
	 * booleano.
	 * @return Objeto BooleanElement con el valor entero de esa cadena.
	 */
	public static BooleanElement groundFromString(String sval)
	{
	    return groundFromValue(Boolean.valueOf(sval));
	}
	
	/**
	 * En el caso de las StringPosition ground, devolvemos una cadena con su valor
	 * (p.ej. "false"), y en el caso de las no-ground, devolvemos una cadena no muy
	 * expresiva (por no gastar espacio almacenando un nombre).
	 */
	public String toString()
	{
	    if ( isGround() )
	        return ""+value;
	    else
	        return "<nonground boolean>";
	}
	
	/**
	 * Devuelve el cdigo Java para que los booleanos ground aparezcan en expresiones.
	 */
	public String getExpressionCode()
	{
	    if ( ground )
	        return "BooleanElement.groundFromValue("+value+")";
	    else
	        return null;
	}
	
	/**
	 * Equals hace por dentro una comparacin de identidad, ya que las instancias
	 * ground estn canonicalizadas.
	 */
	public boolean equals ( Object o )
	{
		return ( o != null && this == o );
	}
	
	
	
	
}
