
/**
 * NO ME CONVENCE. HACER OTRA.
 */


package element.fs;

import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;

import element.SimpleElement;
import element.StringPosition;

interface FSVarOrValue extends Cloneable
{
	public boolean isUndefined();
	public boolean isDefined();
	public String getValueAsString();
	public Object clone();
}

class FSValue
{
	
	private String value;

	public String getValueAsString() 
	{
		return value;
	}

	public void setTo(String value) 
	{
		this.value = value;
	}
	
	public boolean isUndefined()
	{
		return ( value == null );
	}
	
	public boolean isDefined()
	{
		return ( value != null );
	}
	
	public Object clone()
	{
		FSValue clone = new FSValue();
		clone.value = this.value;
		return clone;
	}
	
}

class FSVariable
{
	
	private FSValue value;
	
	public boolean isUndefined()
	{
		return ( value == null ) || value.isUndefined();
	}
	
	public boolean isDefined()
	{
		return ( value != null ) && value.isDefined();
	}
	
	public FSValue getValue()
	{
		return value;
	}
	
	public String getValueAsString()
	{
		if ( value == null ) return null;
		else return value.getValueAsString();
	}
	
	public void setValue ( FSValue value )
	{
		this.value = value; 
	}
	
	public Object clone()
	{
		FSVariable clone = new FSVariable();
		clone.value = (FSValue) this.value.clone();
		return clone;
	}
	
}

public class SimpleFeatureStructure extends SimpleElement
{

	private Map featuresToVarsAndValues = new TreeMap();
	
	private boolean ground;
	
	private static java.util.HashMap nonGroundHash = new java.util.HashMap(); //keys: String
	

	public static SimpleFeatureStructure nonGroundFromString(String s)
	{
	    if ( nonGroundHash.get(s) != null )
	        return (SimpleFeatureStructure) nonGroundHash.get(s);
	    else
	    {
	        //nueva
	        SimpleFeatureStructure nueva = new SimpleFeatureStructure(false);
	        nonGroundHash.put(s,nueva);
	        return nueva;
	    }
	}
	
	public SimpleFeatureStructure ( boolean ground ) 
	{
		this.ground = ground;
	}
	
	public SimpleFeatureStructure ( SimpleFeatureStructure toCloneFrom )
	{
		this.ground = toCloneFrom.ground;
		Iterator it = toCloneFrom.featuresToVarsAndValues.keySet().iterator();
		while ( it.hasNext() )
		{
			String feature = (String) it.next();
			FSVarOrValue vov = toCloneFrom.getMapEntry ( feature );
			this.featuresToVarsAndValues.put( feature , vov.clone() ); //deep copy
		}
	}
	
	public Iterator features()
	{
		return featuresToVarsAndValues.keySet().iterator();
	}
	
	public boolean isDefined ( String feature )
	{
		FSVarOrValue vov = (FSVarOrValue) featuresToVarsAndValues.get(feature);
		if ( vov == null ) return false;
		else return vov.isDefined();
	}
	
	public boolean isDefinedOrLinked ( String feature )
	{
		FSVarOrValue vov = (FSVarOrValue) featuresToVarsAndValues.get(feature);
		if ( vov == null ) return false;
		else if ( vov instanceof FSVariable ) return true;
		else return vov.isDefined();
	}
	
	public String getValue ( String feature )
	{
		FSVarOrValue vov = getMapEntry(feature);
		if ( vov == null ) return null;
		else return vov.getValueAsString();
	}
	
	public FSVarOrValue getMapEntry ( String feature )
	{
		return (FSVarOrValue) featuresToVarsAndValues.get(feature);
	}
	
	public void linkToVariable ( String feature , FSVariable var )
	{
		featuresToVarsAndValues.put ( feature , var );
	}
	
	public void setValue ( String feature , String value )
	{
		FSVarOrValue vov = (FSVarOrValue) featuresToVarsAndValues.get(feature);
		if ( vov == null )
		{
			FSValue val = new FSValue();
			val.setTo ( value );
			featuresToVarsAndValues.put ( feature , value );
		}
		else if ( vov instanceof FSVariable )
		{
			FSValue val = new FSValue();
			val.setTo ( value );
			((FSVariable)vov).setValue ( val );
		}
		else if ( vov instanceof FSValue )
		{
			System.err.println("Error (internal): Are you sure this should happen?");
			((FSValue)vov).setTo(value);
		}
		else
		{
			System.err.println("Error (internal): unexpected class on feature structure map!");
		}
	}
	
	public boolean unifies ( SimpleFeatureStructure anotherFS )
	{		
		Iterator it = anotherFS.features();
		while ( it.hasNext() )
		{
			String feature = (String) it.next();
			if ( this.isDefined(feature) && this.getValue(feature) != anotherFS.getValue(feature) )
				return false;
		}
		return true;	
	}
	
	//PRE: unifies() returns true
	public SimpleFeatureStructure unify ( SimpleFeatureStructure anotherFS )
	{
		SimpleFeatureStructure result = new SimpleFeatureStructure(this);
		Iterator it = anotherFS.features();
		while ( it.hasNext() )
		{
			String feature = (String) it.next();
			if ( anotherFS.isDefined(feature) )
				this.setValue( feature , anotherFS.getValue(feature) ); //already sets var if present
		}
		return result;
	}

	//PRE: unifies() returns true
	public SimpleFeatureStructure unifyKeepingMyVariablesOnly ( SimpleFeatureStructure anotherFS )
	{
		SimpleFeatureStructure result = new SimpleFeatureStructure(this);
		Iterator it = anotherFS.features();
		while ( it.hasNext() )
		{
			String feature = (String) it.next();
			FSVarOrValue vov = this.getMapEntry(feature);
			if ( vov != null && vov instanceof FSVariable )
			{
				FSValue newVal = new FSValue();
				newVal.setTo(anotherFS.getValue(feature));
				((FSVariable)vov).setValue(newVal);
			}
		}
		return result;
	}
	
	public boolean isGround()
	{
		return ground;
	}
	
}
