package element.predicate;

import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;



/**
 * Clase que sirve de ayuda para interpretar cadenas de tipo f(a,b,c...) donde
 * los argumentos de la funcin pueden ser funciones a su vez.
 * @author Carlos
 *
 */
public class FunctionExpressionParser 
{

	private String functionName;
	private List arguments;
	
	private char argumentSep = ';';
	
	public FunctionExpressionParser ( String s ) throws FunctionExpressionParseException
	{
		
		//separate function name from arguments string
		StringTokenizer st = new StringTokenizer ( s , "(" , true );
		
		functionName = st.nextToken();
			if ( !st.hasMoreTokens() ) //parntesis izquierdo
			{
				throw new FunctionExpressionParseException("Missing parenthesis at function expression " + s);
			}
			st.nextToken();
			StringBuffer rest = new StringBuffer();
			while ( st.hasMoreTokens() )
			{
				rest.append(st.nextToken());
			}
			if ( rest.charAt(rest.length()-1) != ')' )
			{
				throw new FunctionExpressionParseException("Missing right parenthesis to end function expression " + s);
			}
			String restString = rest.substring(0,rest.length()-1);
			//restString should be the comma-separated argument list.
			
			int nestLevel = 0;
			int nextTokenStart = 0;
			arguments = new ArrayList();
			for ( int i = 0 ; i < restString.length() ; i++ )
			{
				char c = restString.charAt(i);
				if ( c == argumentSep && nestLevel == 0 )
				{
					arguments.add(restString.substring(nextTokenStart,i).trim());
					nextTokenStart = i+1;
				}
				if ( c == '(' )
					nestLevel++;
				if ( c == ')' )
				{
					if ( nestLevel == 0 )
						throw new FunctionExpressionParseException("Too many right parentheses at function expression " + s);
					nestLevel--;	
				}
			}
			if ( nestLevel == 0 )
			{
				arguments.add(restString.substring(nextTokenStart,restString.length()).trim());
			}
			else
			{
				throw new FunctionExpressionParseException("Too many left parentheses at function expression " + s);
			}
			

		
	}
	
	/**
	 * Returns the function name of the parsed expression as a string.
	 * @return Function name that has been parsed.
	 */
	public String getFunctionName()
	{
		return functionName;
	}
	
	/**
	 * Returns a list of the arguments as different strings.
	 * @return A list with each one of the comma-separated arguments of the expression.
	 */
	public List getArguments()
	{
		return arguments;
	}
	
}
