/*
 * Created at regulus on 20-ago-2004 12:33:20
 * as file ExpressionElement.java on package element
 */
package element;

import generator.DeductiveStepCodeGenerator;

/**
 * @author carlos
 * <p>Las instancias de ExpressionElement son elementos de los esquemas de
 * anlisis sintctico que representan expresiones.</p>
 * <p>Dicho de otro modo, son elementos de los esquemas que estn expresados
 * utilizando otros elementos; pero que finalmente deben entenderse como un elemento
 * simple. Por ejemplo, la suma de posiciones de cadena de entrada i+j o i+1, o una 
 * expresin como P*Q, A U P, tail(L), head(L), merge(P,Q), etc.</p>
 *
 * Created at regulus, 20-ago-2004 12:33:20
 */
public abstract class ExpressionElement extends CompositeElement 
{
	
	protected Element[] children;
	
	/**
	 * Devuelve el nmero de elementos hijos de esta expresin.
	 * Este mtodo no ser utilizado por el generador de cdigo,
	 * es decir, no es necesario que el usuario lo implemente si
	 * no lo va a utilizar l mismo.
	 */
	public int numChildren() 
	{
		return children.length;
	}

	/**
	 * Devuelve el i-simo hijo de esta expresin, que puede ser una
	 * subexpresin o bien un elemento simple.
	 * Este mtodo no ser utilizado para el generador de cdigo,
	 * es decir, no es necesario que el usuario lo implemente si
	 * no lo va a utilizar l mismo.
	 */
	public Element getChild(int i) 
	{
		return children[i];
	}
	
	/**
	 * Construye un elemento representando una expresin con los hijos
	 * dados.
	 * @param children Nodos hijos de la expresin, que sern subexpresiones
	 * o elementos.
	 */
	public ExpressionElement ( Element[] children )
	{
		this.children = children;
	}
	
	/**
	 * Soporte para el patrn de diseo Visitante.
	 */
	public void accept ( ElementVisitor v )
	{
		v.visitExpressionElement(this);
	}
	
	//ntese que en esta clase ser muy til CompositeElement#getLeafElements().
	
	/**
	 * Devuelve el cdigo Java que representa una expresin que se evala a un
	 * elemento simple correspondiente al resultado de esta expresin, siendo
	 * memberNames los nombres de expresiones que se evalan a cada uno (por orden)
	 * de los miembros de esta expresin.
	 * @param memberNames Expresin para cada uno de los miembros de la expresin.
	 * @return Expresin en cdigo Java que representa el resultado de aplicar
	 * esta expresin a sus miembros.
	 */
	public abstract String getExpressionCode ( String[] memberNames );
	
	/**
	 * Hace lo mismo que getExpressionCode ( String[] ); pero se le pasa el objeto
	 * generador de cdigo por si es necesario obtener el estado de atributos (cosa
	 * que se hace, por ejemplo, cuando se generan rboles cuyos nodos contienen
	 * reglas). Por defecto llamar a la implementacin de getExpressionCode que
	 * no usa el generador.
	 * @param memberNames Expresin para cada uno de los miembros de la expresin.
	 * @param cg Objeto que se est usando para generar el cdigo y que contiene
	 * el estado de los atributos.
	 * @return Expresin en cdigo Java que representa el resultado de aplicar
	 * esta expresin a sus miembros.
	 */
	public String getExpressionCode ( String[] memberNames , DeductiveStepCodeGenerator cg )
	{
		return getExpressionCode(memberNames);
	}
	
	/**
	 * Devuelve la clase a la que se evala la expresin. Por ejemplo, si la expresin
	 * es una suma de StringPositions, esto deber devolver StringPosition.class.
	 * @return Clase de la expresin, es decir, clase de elemento al que hace referencia
	 * la expresin entendida como elemento simple.
	 */
	public abstract Class getResultingClass();
	
}
