package element;


/**
 * 
 * @author carlos
 *
 * Elemento simple que representa un nmero entero.
 */
public class IntElement extends SimpleElement implements HashIndexable
{
	
	/**
	 * Valor de este entero, que slo tiene sentido para enteros instanciados.
	 */
    protected int value;
	
	/**
	 * Atributo que nos dice si el smbolo es constante/instanciado (ground) o no.
	 */
	protected boolean ground;
	
	/**
	 * Devuelve el propio objeto IntElement.
	 * @see element.HashIndexable#getHashKey()
	 */
	public Object getHashKey() 
	{
		return this;
	}

	/**
	 * Crea un entero con el valor dado.
	 * @param value Valor que tomar el entero, si es ground.
	 * @param ground Si el entero es una constante (ground) o no.
	 */
	public IntElement ( int value , boolean ground )
	{
		this.value = value; this.ground = ground;
	}
	
	private static java.util.HashMap nonGroundHash = new java.util.HashMap(); //keys: String
	private static int valuesForNonGround = 0;
	
	/**
	 * Mtodo de parsing para IntElements, nos da un entero no ground para una cadena
	 * dada. Llamadas con la misma cadena devuelven el mismo objeto.
	 * @param s Cadena que simboliza un smbolo que representa un entero (ej. "#a","#b")
	 * @return Elemento no ground representando un entero correspondiente a la cadena dada.
	 */
	public static IntElement nonGroundIntElement(String s)
	{
		if ( s.charAt(0) == '#')
			s = s.substring(1);
	    if ( nonGroundHash.get(s) != null )
	        return (IntElement) nonGroundHash.get(s);
	    else
	    {
	        //nueva
	        valuesForNonGround++;
	        IntElement nueva = new IntElement(valuesForNonGround,false);
	        nonGroundHash.put(s,nueva);
	        return nueva;
	    }
	}
	
	/**
	 * Devuelve un IntElement constante a partir de una representacin en forma
	 * de cadena de su valor.
	 * @param valString Cadena que representa el valor del elemento deseado,
	 * por ejemplo "#1".
	 * @return Una instancia de IntElement con el valor dado, que puede ser nueva
	 * u otra devuelta anteriormente.
	 */
	public static IntElement groundIntElement ( String valString )
	{
		if ( valString.charAt(0) == '#')
			valString = valString.substring(1);
		int ival = Integer.valueOf(valString).intValue();
		return groundIntElement(ival);
	}
		
	/**
	 * Nos dice si este entero es ground (constante) o no.
	 */
	public boolean isGround()
	{
		return ground;
	}
	
	/**
	 * Devuelve el valor de este entero.
	 */
	public final int getValue()
	{
		return value;
	}
	
	/**Los n primeros enteros se guardan en una "pool".*/
	private static int SHARED_INSTANCES = 50;
	
	private static IntElement[] commonInstances; 
	
	static
	{
		commonInstances = new IntElement[SHARED_INSTANCES];
		for ( int i = 0 ; i < SHARED_INSTANCES ; i++ )
			commonInstances[i] = new IntElement(i,true);
	}
	
	/**
	 * Devuelve un objeto de tipo IntElement ground con el valor dado. Diferentes llamadas
	 * a este mtodo con el mismo valor pueden o no devolver la misma instancia.
	 * @param value Valor entero.
	 * @return Una instancia de IntElement con el valor dado, que puede ser la misma
	 * ya devuelta en una ocasin anterior.
	 */
	public static IntElement groundIntElement ( int value )
	{
		if ( value < SHARED_INSTANCES && value >= 0 )
			return commonInstances[value];
		else
			return new IntElement(value,true);
	}
		
	/**
	 * Devuelve el cdigo Java para que las posiciones ground aparezcan en expresiones.
	 */
	public String getExpressionCode()
	{
	    if ( ground )
	    	return "IntElement.groundIntElement("+value+")";
	    else
	        return null;
	}
	
	/**
	 * En el caso de las IntElement ground, devolvemos una cadena con su valor
	 * (p.ej. "3"), y en el caso de las no-ground, devolvemos un entero prefijado con
	 * una j (j1, j2, etc.)
	 */
	public String toString()
	{
	    if ( isGround() )
	        return ""+value;
	    else
	        return "j"+value;
	}
	
	
	/**
	 * Comparacin de igualdad: al no estar los IntElement canonicalizados, no nos
	 * vale la comparacin por defecto, que es de identidad de objetos. Tenemos que
	 * definir IntElements iguales como aquellos que tienen la misma "groundness" y
	 * el mismo valor entero asociado. Para que esto funcione con elementos no ground,
	 * debemos instanciarlos todos con el nonGroundIntElement(), que le da a cada
	 * uno un valor distinto.
	 * @param o Objeto con que comparamos ste.
	 * @return Si ambos objetos son iguales o no.
	 */
	
	/**
	 * Comparacin de igualdad de enteros.
	 */
	public boolean equals ( Object o )
	{
		if ( o == null ) return false;
		if ( o.getClass() == this.getClass() )
		{
			IntElement ie = (IntElement)o;
			return ie.value == this.value && ie.ground == this.ground;
		}
		else
		{
			return false;
		}
	}
	
	
	
	
	
}