package element.fs;

import parser.eparser.ElementParseException;
import parser.eparser.ElementParser;

import java.util.List;

import element.Element;
import element.ExpressionElement;
import element.predicate.FunctionExpressionParseException;
import element.predicate.FunctionExpressionParser;

/**
 * Represents an expression of the form Top-features(node) which gets
 * the given node's associated top feature structure from the grammar.
 * @author Carlos
 *
 */
public class BottomFeaturesExpression extends ExpressionElement
{

		public BottomFeaturesExpression(Element argument)
		{
			super( new Element[] { argument } );
		}
		
		public boolean isGround()
		{
			return false;
		}
		
		public Class getResultingClass()
		{
			return PlainFeatureStructure.class;
		}
		
		public String getExpressionCode(String[] memberNames) 
		{			
			StringBuffer sb = new StringBuffer("");
			sb.append("((grammar.tag.TreeAdjoiningGrammarWithFeatureStructures)");
			sb.append(memberNames[0]);
			sb.append(".getAssociatedTree().getGrammar()).getBottomFeatureStructure(");
			sb.append(memberNames[0]);
			sb.append(")");
			return new String(sb);
		}
		
		public static BottomFeaturesExpression fromString ( String sparm ) throws ElementParseException
		{
			try
			{
				FunctionExpressionParser fep = new FunctionExpressionParser(sparm);
				List expressionArgs = fep.getArguments();
				if ( expressionArgs.size() != 1 )
					throw new ElementParseException("Bottom Features expression taking " + expressionArgs.size() + " arguments when 1 are expected");
				Element argument = ElementParser.getCurrentInstance().parse(((String)expressionArgs.get(0)));
				return new BottomFeaturesExpression ( argument );
				
			}
			catch ( FunctionExpressionParseException fepe )
			{
				throw new ElementParseException(fepe);
			}
		}

		
}
