package element.fs;

import java.util.List;

import parser.eparser.ElementParseException;
import parser.eparser.ElementParser;
import element.BooleanElement;
import element.Element;
import element.ExpressionElement;
import element.predicate.FunctionExpressionParseException;
import element.predicate.FunctionExpressionParser;

/**
 * Esta clase representa una expresin booleana de tipo Unifies(fs1;fs2)
 * que nos dice si una estructura de rasgos unifica con otra.
 * @author Carlos
 */
public class UnifiesExpression extends ExpressionElement 
{

	public UnifiesExpression(Element[] arguments)
	{
		super( arguments );
	}
	
	public boolean isGround()
	{
		return false;
	}
	
	public Class getResultingClass()
	{
		return BooleanElement.class;
	}
	
	 /**
     * La expresin Adjoinable se resuelve al cdigo Java que llama a un mtodo
     * que determina si un rbol es adjuntable en un nodo.
     */
	public String getExpressionCode(String[] memberNames) 
	{
		StringBuffer result = new StringBuffer();
		
		result.append("BooleanElement.groundFromValue(");
		for ( int lastarg = 1 ; lastarg < memberNames.length ; lastarg++ )
		{
			
			//generate 1.unify(2).unify(3).unify(4).(...).unifies(lastarg) expression
			
			//generate the unify part
			result.append(memberNames[0]);
			for ( int i = 1 ; i < lastarg ; i++ )
			{
	        	result.append(".unify( ");
	        	result.append(memberNames[i]);
	        	result.append(" )");
			}
			
			//generate the unifies part
			result.append(".unifies(");
			result.append(memberNames[lastarg]);
			result.append(" )");
			
			
			if ( lastarg < memberNames.length - 1 )
				result.append(" && ");
			
		}
		result.append(")");
        return result.toString();
	}
	
	public static UnifiesExpression fromString ( String sparm ) throws ElementParseException
	{

		try
		{
			FunctionExpressionParser fep = new FunctionExpressionParser(sparm);
			
			List expressionArgs = fep.getArguments();
			
			if ( expressionArgs.size() < 2 )
			{
				throw new ElementParseException("Unifies expression taking " + expressionArgs.size() + " arguments when 2 or more are expected");
			}
			
			Element[] arguments = new Element[expressionArgs.size()];
			for ( int i = 0 ; i < arguments.length ; i++ )
			{
				arguments[i] = ElementParser.getCurrentInstance().parse((String)expressionArgs.get(i));
			}
			
			return new UnifiesExpression ( arguments );
			
		}
		catch ( FunctionExpressionParseException fepe )
		{
			throw new ElementParseException(fepe);
		}
		
	}
	
}
