package element.predicate;

import java.util.List;

import parser.eparser.ElementParseException;
import parser.eparser.ElementParser;
import element.BooleanElement;
import element.Element;
import element.ExpressionElement;
import element.SumOfIntegersExpression;

/**
 * Represents a boolean expression of the form union(p1;p2) where p1 and p2
 * are nullable string positions.
 */
public class MakeNullableExpression extends ExpressionElement 
{

	public MakeNullableExpression(Element argument1 )
	{
		super( new Element[] { argument1 } );
	}
	
	public boolean isGround()
	{
		return false;
	}
	
	public Class getResultingClass()
	{
		return element.NullableStringPosition.class;
	}
	
	public String getExpressionCode(String[] memberNames) 
	{
        StringBuffer result = new StringBuffer("");
        result.append(memberNames[0]);
        result.append(".nullable()");
        return result.toString();
	}
	
	public static MakeNullableExpression fromString ( String sparm ) throws ElementParseException
	{

		try
		{
			FunctionExpressionParser fep = new FunctionExpressionParser(sparm);
			
			List expressionArgs = fep.getArguments();
			
			if ( expressionArgs.size() != 1 )
			{
				throw new ElementParseException("Make Nullable expression taking " + expressionArgs.size() + " arguments when 1 is expected");
			}
			
			Element argument1 = ElementParser.getCurrentInstance().parse(((String)expressionArgs.get(0)));
			
			return new MakeNullableExpression ( argument1 );
			
		}
		catch ( FunctionExpressionParseException fepe )
		{
			throw new ElementParseException(fepe);
		}
		
	}
	
}
