package element.tag.predicate;

import java.util.List;

import parser.eparser.ElementParseException;
import parser.eparser.ElementParser;
import element.BooleanElement;
import element.Element;
import element.ExpressionElement;
import element.SumOfIntegersExpression;
import element.predicate.FunctionExpressionParseException;
import element.predicate.FunctionExpressionParser;

/**
 * Represents a boolean expression of the form Selects-tree ( word ; tree ) which returns
 * true if the given word selects the given tree.
 * @author Carlos
 *
 */
public class SelectsTreeExpression extends ExpressionElement 
{

	public SelectsTreeExpression(Element argument1 , Element argument2)
	{
		super( new Element[] { argument1 , argument2 } );
	}
	
	public boolean isGround()
	{
		return false;
	}
	
	public Class getResultingClass()
	{
		return BooleanElement.class;
	}
	
	public String getExpressionCode(String[] memberNames) 
	{
		
		StringBuffer sb = new StringBuffer("");
		sb.append("BooleanElement.groundFromValue(");
		sb.append("(((element.fs.WordFeaturesExpression.getWordInformation(");
		sb.append(memberNames[0]);
		sb.append(".getName()");
		sb.append("))[0]).selectsTree(");
		sb.append(memberNames[1]);
		sb.append("))");
		sb.append(")");
        return sb.toString();
        
	}
	
	public static SelectsTreeExpression fromString ( String sparm ) throws ElementParseException
	{

		try
		{
			FunctionExpressionParser fep = new FunctionExpressionParser(sparm);
			
			List expressionArgs = fep.getArguments();
			
			if ( expressionArgs.size() != 2 )
			{
				throw new ElementParseException("Selects Tree expression taking " + expressionArgs.size() + " arguments when 2 are expected");
			}
			
			Element argument1 = ElementParser.getCurrentInstance().parse(((String)expressionArgs.get(0)));
			Element argument2 = ElementParser.getCurrentInstance().parse(((String)expressionArgs.get(1)));
		
			return new SelectsTreeExpression ( argument1 , argument2 );
			
		}
		catch ( FunctionExpressionParseException fepe )
		{
			throw new ElementParseException(fepe);
		}
		
	}
	
}
