/*
 * Created on 09-ago-2004
 *
 */
package generator;

import element.CompositeElement;
import element.SimpleElement;
import element.ExpressionElement;
import element.SequenceElement;

/**
 * 
 * <p>Genera el cdigo del chequeo inicial del mtodo applyTo(), que es un chequeo
 * estructural del rbol que no incluye comprobacin de clases.</p>
 * <p>Created 09-ago-2004 11:38:01</p>
 * @author carlos
 */
public class ApplyInitialCheckVisitor extends CodeGeneratingVisitor 
{

	/**
	 * @param objectName Nombre del objeto que analizamos en el cdigo.
	 * @param toAppendTo Buffer al que se agregar el cdigo generado.
	 */
	public ApplyInitialCheckVisitor(String objectName, StringBuffer toAppendTo , DeductiveStepCodeGenerator cg) 
	{
		super(objectName, toAppendTo,cg);
	}

	/**
	 * Genera el cdigo para un elemento simple, que consiste en... nada!
	 */
	public void visitSimpleElement(SimpleElement elt) 
	{
		;
	}
	
	/**
	 * Genera el cdigo para un elemento de expresin, que consiste en... nada!
	 */
	public void visitExpressionElement(ExpressionElement elt)
	{
	    ;
	}
	
	/**
	 * Genera el cdigo para un elemento de tipo Sequence, que consiste en... nada!
	 */
	public void visitSequenceElement(SequenceElement zome)
	{
		;
		/**
		 * Ntese que las cuentas de longitud de los SequenceElements
		 * para el chequeo estructural las lleva el CompositeElement.
		 */
	}
	
    /**Flag que slo se activa en la primera llamada al visitante, que ser la ms externa, despus estar a false
     * para que el resto de llamadas recursivas. Esto se utiliza porque en las llamadas
     * que no sean la ms externa harn falta casts a Object[] para los elementos composite,
     * el ms externo ya sabemos que es un Object[] y por optimizar no hacemos tal
     * cast, que sera innecesario.*/
    private boolean firstTimeFlag = true;
	
	/**
	 * Genera el cdigo para un elemento compuesto, que consiste en comprobar que la longitud
	 * del array de objetos que analizamos en el cdigo coincide con la longitud esperada
	 * (que es el nmero de hijos de nuestro elemento compuesto), y despus generar el cdigo
	 * para cada uno de los elementos del array y los hijos correspondientes (para ello
	 * el nombre de objeto de cdigo referenciado por el visitante se va cambiando a objeto[i]
	 * a medida que se visitan los hijos).
	 */
	public void visitCompositeElement(CompositeElement ce)
	{
		
		String oldObjectName = this.objectName;
		
		//cast to Object[]
		this.objectName = "((Object[])" + oldObjectName + ")";
		
		//check desired length
		//la longitud mnima deseada es el nmero de elementos del composite que
		//no son Sequence
		int desiredLength = ce.numChildren();
		for ( int i = 0 ; i < ce.numChildren() ; i++ )
		{
			if ( ce.getChild(i) instanceof SequenceElement )
				desiredLength--;
		}
		
		boolean externalFlag = firstTimeFlag;
		firstTimeFlag = false;
		
		if ( !externalFlag )
		{
			toAppendTo.append ( "if ( " + oldObjectName + " instanceof Object[] )\n");
			toAppendTo.append("{\n");
		}	
		
		if ( desiredLength == ce.numChildren() )
			toAppendTo.append ( "if ( " + objectName + ".length != " + ce.numChildren() + " ) return result;\n" );
		else
			toAppendTo.append ( "if ( " + objectName + ".length < " + desiredLength + " ) return result;\n" );
		
		if ( !externalFlag )
		{
			toAppendTo.append("} //class is Object[]\n");
	        toAppendTo.append("else\n");
	        toAppendTo.append("{\n");
	        toAppendTo.append ( "return result;\n" );
	        toAppendTo.append ( "} //class is not Object[]\n" ); 	
		}
		
		for ( int i = 0 ; i < ce.numChildren() ; i++ )
		{
			//this.objectName = "((Object[])" + oldObjectName + "["+i+"])";
			this.objectName = oldObjectName + "["+i+"]";
			ce.getChild(i).accept(this);
		}
		this.objectName = oldObjectName; //dejamos el visitante como estaba.
	}

}
