/*
 * Created on 10-ago-2004 11:29:01
 *
 */
package generator;

import element.CompositeElement;
import element.SimpleElement;
import element.ExpressionElement;
import element.SequenceElement;

/**
 * <p>Genera el cdigo que crea la especificacin de bsqueda del item que buscamos para pasrselo
 * a la funcin que maneja los ndices.</p>
 * <p>Created 10-ago-2004 11:29:01</p>
 * @author carlos
 */
public class ApplySpecificationVisitor extends CodeGeneratingVisitor
{

    
    /**
     * Este rbol representa la estructura de la especificacin de bsqueda que hemos generado,
     * en forma de descriptor de bsqueda.  
     * Esto lo recibir la clase que genere el cdigo de la ItemHandler, lo que podramos
     * llamar el "optimizador" que decide qu indices implementar.
     */
    private SearchDescriptorNode skeletonTree;
    
    /**
     * Nodo del descriptor de bsqueda (skeletonTree) que se est generando.
     */
    private SearchDescriptorNode currentNode;
    
	/**
	 * Genera nmeros para aadir al nombre de las variables Object[] que se utilizan
	 * en la creacin de la especificacin.
	 */
	private static int tempVariableIndex = 0;
	
	/**
	 * Devuelve un nombre para una variable temporal. Se garantiza que en cada
	 * llamada se devuelve un nombre distinto.
	 * @return El nombre para una variable temporal.
	 */
	private String nextTempVariableName()
	{
		return ( "skel_temp_" + (tempVariableIndex++) );
	}
	
    /**Flag que slo se activa en la primera llamada al visitante, que ser la ms externa,
     * para poner un "Object[] tal = " al principio y un ";\n" al final. Despus estar a false
     * para que el resto de llamadas recursivas no pongan declaraciones y "\n"'s.*/
    private boolean firstTimeFlag = true;
    
    /**
     * @param objectName En este caso el objectName indicar el nombre que se dar al
     * array especificacin de bsqueda.
     * @param toAppendTo Buffer al que aadiremos el cdigo generado.
     * @param cg Generador de cdigo que invoca al visitador y que tendr la informacin
     * correspondiente a atributos.
     */
    public ApplySpecificationVisitor ( String objectName, StringBuffer toAppendTo, DeductiveStepCodeGenerator cg )
    {
        super ( objectName, toAppendTo, cg );
    }

    /**
     * Como precondicin que se ha de cumplir, un SimpleElement visitado por este objeto
     * siempre estar dentro de un CompositeElement que estemos visitando. Por ello, en
     * el buffer de cdigo, habr una llave de inicializacin de array abierta.
     * Para elementos simples que tengan un valor definido, lo que hacemos es meter ese
     * valor (el atributo correspondiente) en el array. Para los elementos simples que
     * no tengan valor (no-ground), pasamos un nulo.
     */
    public void visitSimpleElement ( SimpleElement se )
    {
        
        Attribute attr = generator.attributeFor(se);
        
        //poner coma si no es el primer atributo de este array
        if ( toAppendTo.length() >= 2 && ( toAppendTo.charAt(toAppendTo.length()-2) != '{' && toAppendTo.charAt(toAppendTo.length()-2) != '=' ) )
            toAppendTo.append(", ");
        
        if ( se.isGround() ) //initially ground
        {
            currentNode = new SearchDescriptorNode ( se.getClass() );
            toAppendTo.append ( se.getExpressionCode() );
        }
        else if ( generator.isGround(attr) ) //grounded
        {
            currentNode = new SearchDescriptorNode ( attr.getElementClass() );
            toAppendTo.append ( attr.getName() );
        }
        else //yet nonground
        {
            currentNode = new SearchDescriptorNode();
            toAppendTo.append ( "null" );
        }
    
    }
    
    /**
     * Con las expresiones la precondicin es la misma que con los elementos simples,
     * y adems hacemos tambin lo mismo, con la diferencia de que si la expresin
     * es ground metemos el cdigo generado para ella en el slot del array (en lugar
     * de un nombre de atributo, como en el caso del elemento simple). En el caso
     * de que la expresin sea no ground, pasamos igualmente un nulo, pues no hay
     * valor que utilizar en el esqueleto para pedir items al ndice.
     * @param expr Nodo correspondiente a una expresin.
     */
    public void visitExpressionElement ( ExpressionElement expr )
    {
        
        //poner coma si no es el primer atributo de este array
        if ( toAppendTo.charAt(toAppendTo.length()-2) != '{' )
            toAppendTo.append(", ");
        
        if ( generator.isGround(expr) )
        {
            //ground or grounded
            currentNode = new SearchDescriptorNode ( expr.getResultingClass() );
            toAppendTo.append( generator.generateExpressionCode(expr) );
        }
        else //expression is nonground
        {
            currentNode = new SearchDescriptorNode();
            toAppendTo.append ( "null" );
        }
    }

    /**
     * La especificacin de bsqueda para un Sequence no ground va a ser un nodo asociado a la clase
     * Sequence, dado que nos interesa indicar que el rbol se puede prolongar
     * por ah.
     * En el caso de los Sequence ground, se comportarn como se comportaran
     * sus componentes si estuviesen aadidos al padre del Sequence. (i.e. un
     * Sequence ground no crea un nuevo nivel ni nodo en la especificacin de bsqueda; sino
     * que sus hijos van al mismo nivel que sus hermanos).
     */
    
    public void visitSequenceElement ( SequenceElement zome )
    {
    	
    	Attribute attr = generator.attributeFor(zome);
    	        
        if ( zome.isGround() ) //initially ground
        {
        	//add children directly to array structure in code
        	//(hanging from the Sequence's parent)
            for ( int i = 0 ; i < zome.numChildren() ; i++ )
            {
            	zome.getChild(i).accept(this);
            }
            //node representing that zero or more elements go in the request
            //is a Sequence, NON-LEAF node.
            currentNode = new SearchDescriptorNode( new SearchDescriptorNode[]{ new SearchDescriptorNode(zome.getAssociatedClass()) } );
            currentNode.setAssociatedClass(SequenceElement.class);
        }
        else if ( generator.isGround(attr) ) //grounded: the same as initially ground.
        {
        	//add children directly to array structure in code
        	//(hanging from the Sequence's parent):
        	
        	//This is handled when processing the parent via visitCompositeElement.
             	
            //node representing that zero or more elements go in the request
            //is a Sequence, NON-LEAF node.
            currentNode = new SearchDescriptorNode( new SearchDescriptorNode[]{ new SearchDescriptorNode(zome.getAssociatedClass()) } );
            currentNode.setAssociatedClass(SequenceElement.class);
        }
        else //yet nonground
        {
        	
            //poner coma si no es el primer atributo de este array
            if ( toAppendTo.length() >= 2 && ( toAppendTo.charAt(toAppendTo.length()-2) != '{' && toAppendTo.charAt(toAppendTo.length()-2) != '=' ) )
                toAppendTo.append(", ");
        	
            currentNode = new SearchDescriptorNode( new SearchDescriptorNode[]{ new SearchDescriptorNode(zome.getAssociatedClass())});
            currentNode.setAssociatedClass(SequenceElement.class);
        	toAppendTo.append ( "SequenceElement.fromClass("+zome.getAssociatedClass().getName()+".class)" );
        }
    	
    	
    }
    
    
        
    /**
     * El esqueleto para un elemento compuesto va a ser un array de objetos con el esqueleto
     * para cada uno de los componentes.
     */
    public void visitCompositeElement ( CompositeElement ce )
    {
        
    	
        boolean externalFlag = firstTimeFlag; //si tenemos que poner un \n al final y la declaracin
        firstTimeFlag = false;
        
        StringBuffer prevCode = new StringBuffer("");
        StringBuffer allCode = new StringBuffer("");
        
        SearchDescriptorNode[] treeArray = new SearchDescriptorNode[ce.numChildren()];
        SearchDescriptorNode compositeNode = new SearchDescriptorNode ( treeArray );
        
        //Tenemos que crear un array de objetos con el esqueleto de cada uno de los
        //componentes del composite. La longitud de este array sera el nmero de
        //hijos del composite si no fuese porque un hijo que sea un Sequence ground
        //puede dar lugar a varios elementos.
        
        //Miramos si hay secuencias instanciadas. En este caso la longitud
        //no es conocida en tiempo de generacin de cdigo.
        boolean unknownLength = false;
        for ( int i = 0 ; i < ce.numChildren() ; i++ )
        {
        	if ( ce.getChild(i) instanceof SequenceElement && generator.isGround(generator.attributeFor(ce.getChild(i))))
        	{
        		unknownLength = true;
        		break;
        	}
        }
        
        if ( unknownLength )
        {
        	
        	//Creamos un Object[] con la longitud apropiada.
        	//(unknownLength porque es desconocida en tiempo de generacin de cdigo;
        	//pero lo que podemos es generar las variables que nos dan esa longitud).
        	
        	//Hallamos dicha cadena que nos da la longitud...
        	String lengthVar = "";
        	int lengthVarIncrement = 0;
        	for ( int i = 0 ; i < ce.numChildren() ; i++ )
        	{
            	if ( ce.getChild(i) instanceof SequenceElement && generator.isGround(generator.attributeFor(ce.getChild(i))))
            	{
            		lengthVar += ("+"+generator.attributeFor(ce.getChild(i)).getName()+".size()");
            	}
            	else
            	{
            		lengthVarIncrement++;
            	}
        	}
        	lengthVar += ("+"+lengthVarIncrement);
        	
        	//Hallamos el nombre de nuestra variable Object[]...
        	String varName;
        	//if ( externalFlag )
        	//	varName = objectName;
        	//else
        		varName = nextTempVariableName();
        	
        	//Agregamos la declaracin, dejando ya longitud fija...
        	//Ntese que este cdigo se agrega al otro al final
        	//si estamos en el composite externo.
        	prevCode.append( "Object[] " + varName + " = new Object[ " + lengthVar + " ];\n");
        	
        	//Manejaremos el ndice con una variable similar a la utilizada para la longitud.
        	String indexVar = "";
        	int indexVarIncrement = 0;
        	
        	//Recorremos los hijos
        	for ( int i = 0 ; i < ce.numChildren(); i++ )
        	{
            	if ( ce.getChild(i) instanceof SequenceElement && generator.isGround(generator.attributeFor(ce.getChild(i))))
            	{
            		String thisChildLengthVar = (generator.attributeFor(ce.getChild(i)).getName()+".size()"); 
            		prevCode.append( "for ( int i = " /*+ indexVar + "+"*/ + "0" + " ; i < " /*+ indexVar + "+" + "0" + "+"*/ + thisChildLengthVar + " ; i++ )\n");
            		prevCode.append( varName + "[" + indexVar + "+" + indexVarIncrement + "+" + "i ] = " + generator.attributeFor(ce.getChild(i)).getName() + ".get(i);\n");
            		indexVar += thisChildLengthVar;
                    treeArray[i] = new SearchDescriptorNode( new SearchDescriptorNode[]{ new SearchDescriptorNode(((SequenceElement)ce.getChild(i)).getAssociatedClass()) } );
                    treeArray[i].setAssociatedClass(SequenceElement.class);
            	}
            	else
            	{
            		prevCode.append( varName + "[" + indexVar + "+" + indexVarIncrement + " ] = ");
            		StringBuffer temp = this.toAppendTo;
            		this.toAppendTo = prevCode;
            		ce.getChild(i).accept(this);
            		this.toAppendTo = temp;
            		prevCode.append(";\n");
            		treeArray[i] = currentNode;
            		indexVarIncrement++;
            	}
        	}
        	
        	//Este composite es varName
        	if ( externalFlag )
        		allCode.append( "Object[] " + objectName + " = ");
        	
	        //poner coma si no es el primer atributo de este array <new version> [06-11-13]
	        if ( !externalFlag && toAppendTo.length() >= 2 && ( toAppendTo.charAt(toAppendTo.length()-2) != '{' && toAppendTo.charAt(toAppendTo.length()-2) != '=' ) )
	        {
	        	allCode.append(", ");
	        }
        	
        	allCode.append(varName);
        	
        	if ( externalFlag )
        		allCode.append(";\n");

        }
        else //longitud conocida en tiempo de generacin de cdigo
        {
	                
	        if ( externalFlag )
	            allCode.append( "Object[] " + objectName + " = ");
	        //poner coma si no es el primer atributo de este array <old version>
	        //creo que esto no hace nada
	        else if ( allCode.length() >= 2 && allCode.charAt(allCode.length()-2) != '{' )
	            allCode.append(", ");
	        
	        //poner coma si no es el primer atributo de este array <new version> [06-11-13]
	        if ( !externalFlag && toAppendTo.length() >= 2 && ( toAppendTo.charAt(toAppendTo.length()-2) != '{' && toAppendTo.charAt(toAppendTo.length()-2) != '=' ) )
	        {
	        	allCode.append(", ");
	        }
	        	
	        allCode.append ( "new Object[] { " );
	        
	        for ( int i = 0 ; i < ce.numChildren() ; i++ )
	        {
	        	StringBuffer temp = this.toAppendTo;
        		this.toAppendTo = allCode;
	            ce.getChild(i).accept(this);
	            this.toAppendTo = temp;
	            treeArray[i] = currentNode;
	        }
	        
	        allCode.append ( " }");
	        if ( externalFlag )
	            allCode.append(";\n");
	        
        }
        
        currentNode = compositeNode;
        
        if ( externalFlag )
        {
            skeletonTree = currentNode;
            allCode.insert(0,prevCode);
            toAppendTo.append(allCode);
        }
        else //internal composite
        {
        	toAppendTo.insert(0,prevCode);
        	toAppendTo.append(allCode);
        }
        
        
    }

    /**
     * Devuelve lel descriptor de bsqueda que describe la estructura de la especificacin de bsqueda
     * asociado al cdigo generado por este visitante.
     * @return El descriptor de bsqueda asociado, despus de haber generado el cdigo.
     */
    public SearchDescriptorNode getSearchDescriptor()
    {
        return skeletonTree;
    }
    
}
