/*
 * Created at regulus on 06-nov-2004 17:31:05
 * as file AtomicIndex.java on package generator
 */
package generator;

import java.util.List;
import java.util.Iterator;

import element.Element;


/**
 * 
 * <p>Como se explica ms detalladamente en la documentacin de la clase IndexDescriptor,
 * un AtomicIndex representa un ndice de determinado tipo sobre determinada direccin
 * de nodo.</p>
 * <p>Created 12-ago-2004 17:36:35</p>
 * @see IndexDescriptor
 * @author carlos
 */
public class AtomicIndex
{
    
    /**
     * Direccin del nodo por el que se indexa en el formato de lista de Integer
     * descrito en la documentacin de IndexDescriptor.
     * @see IndexDescriptor
     * */
    private List /*of Integer*/ treeNodeAddress;
    
    /**
     * Descriptor de ndice al que pertenece este ndice atmico.
     */
    private IndexDescriptor parent;
    
    /**
     * Tipo de ndice, que puede ser una de las constantes protected static de
     * esta clase.
     */
    private int indexType;
    
    /**
     * Valor de indexType que indica ndice sobre un array.
     */
    protected static final int ARRAY_INDEX = 1;
    
    /**
     * Valor de indexType que indica ndice sobre una tabla hash.
     */
    protected static final int HASH_INDEX = 2;
    
    /**
     * Representa la decisin de indexar por el nodo de direccin addr en algn rbol
     * de esqueleto asociado, mediante un ndice de tipo type.
     * @param addr Direccin del nodo que se indexa.
     * @param type Tipo de ndice.
     */
    public AtomicIndex ( List addr , int type )
    {
        this.treeNodeAddress = addr;
        this.indexType = type;
    }
    
    /**
     * Nos dice si este ndice atmico es igual a otro.
     * Sern iguales si son del mismo tipo y se refieren a la misma direccin de nodo.
     * @param o Objecto con que comparamos.
     */
    public boolean equals ( Object o )
    {
    	if ( !( o instanceof AtomicIndex ) )
    		return false;
    	AtomicIndex ai = (AtomicIndex) o;
    	if ( this.indexType != ai.indexType ) return false;
    	if ( this.treeNodeAddress.size() != ai.treeNodeAddress.size() ) return false;
    	for ( int i = 0 ; i < this.treeNodeAddress.size() ; i++ )
    	{
    		if ( ! this.treeNodeAddress.get(i).equals( ai.treeNodeAddress.get(i) ) )
    			return false;
    	}
    	return true;
    }
    
    /**
     * @return Devuelve el tipo de ndice al que nos referimos.
     */
    public int getIndexType ()
    {
        return indexType;
    }
    /**
     * @return Devuelve la direccin del nodo del rbol de esqueleto a que se refiere
     * este ndice.
     */
    public List getTreeNodeAddress ()
    {
        return treeNodeAddress;
    }
    
    /**
     * Establece el ndice padre de este ndice atmico.
     * @param id
     */
    void setParent ( IndexDescriptor id )
    {
        parent = id;
    }
    
    /**
     * Nos da la clase del elemento por el que estamos indexando. parent tiene que ser != null.
     * @return Clase del elemento por el que indexamos.
     */
    Class getIndexedNodeClass ()
    {
        if ( parent == null ) return null;
        List skelTrees = parent.getAssociatedSearchDescriptors();
        if ( skelTrees.size() < 1 ) return null;
        SearchDescriptorNode skelTree = (SearchDescriptorNode) skelTrees.get(0);
        return skelTree.nodeAtAddress(treeNodeAddress).getAssociatedClass();
    } 
    
    /**
     * Obtiene la direccin del nodo searched en el rbol de raz curNode.
     * Direcciones tipo:
     * epsilon
     *  -> 0
     *  -> 1
     *    -> 1,0
     *    -> 1,1
     *    -> 1,2
     *  -> 2
     * etc.
     * (son listas de Integer)
     * @param curAddress Lista donde se va escribiendo la direccin actual, y donde
     * deseamos que aparezca la direccin del nodo buscado.
     * @param curNode Nodo raz que estamos considerando en este momento, y que
     * tendr raz curAddress. Inicialmente, se pasar la raz del rbol en que queremos
     * buscar y curAddress tendr la lista vaca.
     * @param searched Elemento cuya direccin buscamos.
     * @return Direccin del elemento searched en el rbol, o bien null si
     * el elemento no est.
     */
    private static List getAddress ( List curAddress , Object[] curNode , Element searched )
    {
        for ( int i = 0 ; i < curNode.length ; i++ )
        {
            curAddress.add ( new Integer(i) );
            if ( curNode[i] == searched ) return curAddress;
            else if ( curNode[i] instanceof Object[] )
            {
                curAddress = getAddress(curAddress,(Object[])curNode[i],searched);
            }
            curAddress.remove(curAddress.size()-1); //quitar ltimo
        }
        return null;
    }
    
    //Devuelve una cadena con la direccin por la que indexa este ndice.
    public String toString()
    {
    	StringBuffer sb = new StringBuffer("[");
    	for ( Iterator it = treeNodeAddress.iterator() ; it.hasNext() ; )
    	{
    		Integer num = (Integer) it.next();
    		if ( sb.charAt(sb.length()-1) != '[' )
    			sb.append(",");
    		sb.append(num);
    	}
    	sb.append("]");
    	return sb.toString();
    }
    
}


