/*
 * Created at regulus on 06-nov-2004 17:41:35
 * as file Attribute.java on package generator
 */
package generator;

import element.Element;

/**
 * Representa un atributo del cdigo que generamos para un paso deductivo.
 * 
 * @author carlos
 *
 */
public class Attribute
{
	
	private String name;
	private Element associatedElement;
	private int dimension; /*dimension 0 = no ArrayList, 1 = ArrayList (secuencia).
					Se deja abierta la posibilidad de que hubiese "secuencias multidimensionales"; aunque
					por el momento no existe tal cosa*/
	
	/**
	 * Crea un atributo de nombre dado, asociado al elemento dado.
	 * @param name Nombre que tomar el atributo en el cdigo Java generado.
	 * @param associatedElement Elemento al que est asociado el atributo en la
	 * representacin del "parsing schema" que maneja el generador de cdigo.
	 * @param dimension Tomar el valor cero si el atributo representa un elemento
	 * simple, y el valor uno si representa una secuencia de cero o ms elementos
	 * (es decir, si se corresponde con un elemento tipo ZeroOrMore).
	 */
	public Attribute ( String name , Element associatedElement , int dimension )
	{
		this.name = stripName(name);
		this.associatedElement = associatedElement;
		this.dimension = dimension;
	}
	
	public Attribute ( String name , Element associatedElement )
	{
		this ( name , associatedElement , 0 );
	}
	
	/**
	 * Convierte la cadena dada en un nombre vlido para un atributo Java,
	 * quitndole los smbolos que no sean vlidos en nombres de atributo.
	 * Garantiza que es una aplicacin inyectiva.
	 * @param name
	 * @return
	 */
	private static String stripName ( String name )
	{
		StringBuffer sb = new StringBuffer("");
		if ( !Character.isJavaIdentifierStart(name.charAt(0)) || name.charAt(0)=='_' )
		{
			sb.append("_");
			sb.append(((int)name.charAt(0)));
			sb.append("_");
		}
		else
			sb.append(name.charAt(0));
		for ( int i = 1 ; i < name.length() ; i++ )
		{
			if ( !Character.isJavaIdentifierPart(name.charAt(i)))
			{
				sb.append("_");
				sb.append(((int)name.charAt(i)));
				sb.append("_");
			}
			else
				sb.append(name.charAt(i));
		}
		return new String(sb);
	}
	
	public Attribute ( Element associatedElement )
	{
		String name;
		name = DeductiveStepCodeGenerator.VAR_PREFFIX + stripName(associatedElement.toString());
		this.name = name;
		this.associatedElement = associatedElement;
		this.dimension = 0;
	}
	
	public Attribute ( Element associatedElement , int dimension )
	{
		String name;
		name = DeductiveStepCodeGenerator.VAR_PREFFIX + stripName(associatedElement.toString());
		this.name = name;
		this.associatedElement = associatedElement;
		this.dimension = dimension;
	}
	
	public String getName()
	{
		return name;
	}
	
	public Element getElement()
	{
		return associatedElement;
	}
	
	public String getClassName()
	{
		return associatedElement.getClass().getName();
	}
	
	public Class getElementClass()
	{
		return associatedElement.getClass();
	}
	
	/**
	 * Devuelve el cdigo que declara este atributo, sin inicializarlo.
	 * @return Cadena con cdigo Java para declarar este atributo.
	 */
	public String getDeclaration()
	{
		if ( dimension == 0 )
			return "" + associatedElement.getClass().getName() + " " + name + ";";
		else
			return "" + "ArrayList" + " " + name + " = new ArrayList();";
	}
	
	
}
