/*
 * Created at regulus on 16-feb-2005 16:57:03
 * as file DeductiveStepCodeGenerator.java on package generator
 */
package generator;

import java.util.ArrayList;
import java.util.List;
import java.util.ListIterator;
import java.util.ListResourceBundle;

import element.Element;
import element.ExpressionElement;
import element.GenericWrapperElement;
import element.SequenceElement;
import parser.sparser.tree.AntecedentNode;
import parser.sparser.tree.DeductiveStepNode;
import parser.sparser.tree.ElementNode;
import parser.sparser.tree.ItemNode;
import parser.sparser.tree.ParseTreeNode;

/**
 * @author carlos
 *
 * Created at regulus, 16-feb-2005 16:57:03
 * Esta clase se encarga de generar el cdigo correspondiente a un determinado paso
 * deductivo de un esquema de anlisis sintctico.
 */
public class DeductiveStepCodeGenerator 
{

	//nodo de paso deductivo cuyo cdigo queremos generar
	private DeductiveStepNode dsn;
	
	private StringBuffer attributes; //buffer para cdigo de atributos
	private StringBuffer methods; //buffer para cdigo de mtodos
	
	private java.util.List attrElementList; 
	private java.util.List attrList; //lista de atributos que tendremos
	private java.util.List groundAttrList; //lista de atributos a los que hemos dado un valor
											//"ground" concreto al instanciar el paso
	
	
	 /**
     * Genera nmeros para aadir al nombre de las variables temporales
     * que se generan y garantizar la unicidad de nombres.
     */
    private int tempVariableIndex = 0;
    
    /**
     * Devuelve un nombre para una variable temporal. Se garantiza que en cada
     * llamada se devuelve un nombre distinto.
     * @return El nombre para una variable temporal.
     */
    public String nextItemVariableName()
    {
    	return ( "myItem" + (tempVariableIndex++) );
    }
	

	private class PredicateStateObserver
	{
		
		//para cada predicado:
		//"false" si no se puede evaluar an (expresin no ground)
		//"true" si ya se puede evaluar (expresin ground)
		private List predicateList;
		private List predicateStates;
		private List changed;
		
		private PredicateStateObserver ( )
		{
			ParseTreeNode predicates = dsn.getPredicates();
			predicateList = new ArrayList();
			for ( int i = 0 ; i < predicates.numChildren() ; i++ )
			{
				Element predicate = ((ElementNode) predicates.getChild(i)).getElement();
				predicateList.add(predicate);
			}
			initState();
			initChanged();
			//recalculateState();
		}
		
		private void initState()
		{
			predicateStates = new ArrayList();
			for ( int i = 0 ; i < predicateList.size() ; i++ )
			{
				predicateStates.add ( new Boolean(false) );
			}
		}
		
		private void initChanged()
		{
			changed = new ArrayList();
			for ( int i = 0 ; i < predicateList.size() ; i++ )
			{
				changed.add ( new Boolean(false) );
			}
		}
		
		private void resetChanged()
		{
			for ( int i = 0 ; i < predicateList.size() ; i++ )
			{
				changed.set( i , new Boolean(false) );
			}
		}
		
		private void recalculateState ( )
		{
			for ( int i = 0 ; i < predicateList.size() ; i++ )
			{
				Element predicate = (Element) predicateList.get(i);
				Boolean val;
				if ( predicate instanceof ExpressionElement )
					val = new Boolean( isGround((ExpressionElement)predicate) ) ;
				else
					//this should only happen if a boolean element is directly specified
					//(i.e. the "true" and "false" trivial predicates)
					val = new Boolean( predicate.isGround() ) ;
				if ( !val.equals(predicateStates.get(i)) )
				{
					predicateStates.set(i,val);
					changed.set( i,new Boolean(true) );
				}
			}
		}
		
		private List getChangedPredicates()
		{
			List result = new ArrayList();
			recalculateState();
			for ( int i = 0 ; i < predicateList.size() ; i++ )
			{
				if ( ((Boolean)changed.get(i)).booleanValue() )
				{
					result.add(predicateList.get(i));
				}
			}
			return result;
		}
		
	}

	
	
	/**prefijo para los nombres de las clases generadas*/
	public static String CLASS_PREFFIX = "SP_";
	/**prefijo para los nombres de las variables y atributos generados*/
	public static String VAR_PREFFIX = "sp_";
	/**prefijo para los nombres de los mtodos generados*/
	public static String METHOD_PREFFIX = "sp_";
	
	private String className; //nombre de la clase que generamos
	
	/**Nmero de pasos tipo meta que se han generado. Este contador se mantiene
	 * porque a los pasos meta no les pone nombre el usuario, con lo cual nosotros
	 * les daremos un nombre que incluir un nmero para distinguirlos.*/
	private static int goalsGenerated = 0;
	
	/**
	 * Objeto que lleva cuenta del estado de los predicados para saber cundo
	 * se pueden evaluar.
	 */
	private PredicateStateObserver obs;
	
	/**
	 * @return El nombre de la clase generada por este generador.
	 */
	public String getClassName()
	{
		return className;
	}
	
	/**
	 * Construye un generador de cdigo para un paso deductivo concreto,
	 * dado por el nodo que se pasa como parmetro.
	 * @param dsn Nodo de esquema de anlisis sintctico que representa un
	 * paso deductivo.
	 */
	public DeductiveStepCodeGenerator( DeductiveStepNode dsn ) 
	{	
		this.dsn = dsn;
		this.obs = new PredicateStateObserver();
	}
	
	//visibilidad package
	/**
	 * Devuelve el atributo que se ha definido para un elemento dado; si no existe
	 * tal atributo devuelve null.
	 */
	Attribute attributeFor ( Element elt )
	{
		for ( int i = 0 ; i < attrList.size() ; i++ )
		{
			if ( ((Attribute)attrList.get(i)).getElement().equals(elt) )
				return ((Attribute)attrList.get(i));
		}
		return null;
	}
	
	/**
	 * Indica si el atributo dado tiene asignado un valor concreto en este momento.
	 */	
	boolean isGround ( Attribute attr )
	{
		return groundAttrList.contains(attr);
	}
	
	/**
	 * Nos dice si una expresin dada es "ground" (constante/resuelta), es decir, se puede
	 * sustituir ya por un SimpleElement constante. Para ello, analizar si son constantes
	 * los elementos que forman los nodos hoja de la expresin.
	 * @param ee Elemento que representa una expresin.
	 * @return Si, con el estado actual de los atributos, la expresin
	 * es "ground" o no.
	 */
	boolean isGround ( ExpressionElement ee )
	{
		
		List expressionLeaves = ee.getLeafElements();
		//ver si todas las hojas de la expresin estn resueltas (grounded)
		boolean nonground = false;
		for ( int i = 0 ; i < expressionLeaves.size() ; i++ )
		{
			Element elt = (Element) expressionLeaves.get(i);
			if ( !elt.isGround() && !this.isGround(this.attributeFor(elt)) )
			{
				nonground = true;
				break;
			}
		}
		return !nonground;
	}
	
	/**
	 * Convierte un ExpressionElement, representando una expresin, en una cadena
	 * de cdigo, utilizando para ello el visitante correspondiente.
	 * @param ee Elemento que representa una expresin.
	 * @return Cadena de cdigo representando la expresin, incluyendo los atributos
	 * y constantes necesarios para ello. Ntese que para ello la expresin tiene
	 * que ser ground en este momento desde el punto de vista del generador de cdigo,
	 * cosa que se puede chequear mediante el mtodo isGround() de esta misma clase.
	 * De lo contrario, habr errores.
	 */
	String generateExpressionCode ( ExpressionElement ee )
	{
		StringBuffer sb = new StringBuffer("");
		ExpressionVisitor ev = new ExpressionVisitor(sb,this);
		ee.accept(ev);
		return sb.toString();
	}	
	
	/**
	 * Cambia el valor de "groundness" de un atributo, es decir, establece si se
	 * ha resuelto (tiene un valor constante asignado) o no.
	 */
	void setGround ( Attribute attr , boolean groundVal )
	{
		if ( groundVal )
			groundAttrList.add(attr);
		else
			groundAttrList.remove(attr);
	}
	
	/**
	 * @return La lista de atributos.
	 */
	List getAttrList ( )
	{
		return attrList;
	}
	
	/**
	 * Genera el cdigo asociado a los mtodos, agregndolo al buffer correspondiente
	 * a mtodos que generateCode utilizar en su generacin.
	 * Adems, rellena una lista de SearchDescriptorNodes que contiene la informacin acerca
	 * de las bsquedas que se utilizan en los mtodos para llamar a la funcin
	 * getBySpecification de la clase ItemHandler. Esta informacin se pasar a la clase que
	 * genera el cdigo de ItemHandler.
	 * Asimismo, rellena tambin otra lista de SearchDescriptorNodes que contiene la informacin
	 * acerca de los conjuntos de items con los cuales se puede ejecutar este paso.
	 * La tercera lista de SearchDescriptorNodes que este mtodo debe rellenar es para el mtodo
	 * exists, para comprobar la existencia en la ItemHandler de los items generados en este
	 * paso antes de aadirlos a la misma.
	 * En resumen:
	 * itemSearchSkeletonTrees -> define los conjuntos de items que pediremos a ItemHandler.
	 * stepSearchSkeletonTrees -> define los conjuntos de items que podran activar este
	 * paso deductivo.
	 * existenceSkeletonTrees -> define los conjuntos de items que genera este paso,
	 * por cuya existencia preguntaremos a la ItemHandler.
	 */
	private void generateMethods (  List itemSearchSkeletonTrees , List stepSearchSkeletonTrees , List existenceSkeletonTrees )
	{

		ItemNode condiciones = (ItemNode) dsn.getConditions();
		
		//principio de mtodo constructorPrecondition [para la instanciacin]
		StringBuffer consPredBuffer = new StringBuffer();
		consPredBuffer.append ( "public static boolean constructorPrecondition ( Object[] o )\n" );
		consPredBuffer.append ( "{\n" );
		consPredBuffer.append ( "try\n" );
		consPredBuffer.append ( "{\n" );
		consPredBuffer.append ( "if ( o.length != " + condiciones.numChildren() + " ) return false;\n" );
	
		for ( int i = 0 ; i < condiciones.numChildren() ; i++ )
		{
			ElementNode en = (ElementNode)condiciones.getChild(i);
			
			CodeGeneratingVisitor v = new ConstructorMatchVisitor("o[" + i + "]",consPredBuffer,this);
			en.getElement().accept(v);
		}
				
		obs.resetChanged();
		
		//mtodo constructor. Dejamos el resto de constructorPrecondition() para despus,
		//porque segn los cambios en el estado de "groundness" de los atributos en
		//el constructor puede ser necesario aadirle comprobacin de predicados
		methods.append ( "public " + className + " ( Object[] o )\n" );
		methods.append( "{\n" );
		
		StringBuffer consAssignBuf = new StringBuffer();
		for ( int i = 0 ; i < condiciones.numChildren() ; i++ )
		{
			ElementNode en = (ElementNode)condiciones.getChild(i);
			CodeGeneratingVisitor v = new ConstructorAssignVisitor("o[" + i + "]",consAssignBuf,this);
			en.getElement().accept(v);
		}
		methods.append(consAssignBuf);
		methods.append ( "}\n" );
		//fin de generacin del mtodo constructor
		
		List predGroundAfterConst = obs.getChangedPredicates();
		
		//chequeo de predicados de constructorPrecondition(), si procede
		if ( predGroundAfterConst.size() > 0 )
		{
		
			consPredBuffer.append("{ //predicate-checking block\n");
			
			//declare local variables equivalent to step attributes
			for ( int i = 0 ; i < attrList.size() ; i++ )
			{
				Attribute attr = (Attribute) attrList.get(i);
				consPredBuffer.append(attr.getDeclaration());
				consPredBuffer.append("\n");
			}
			
			//assign values to them just like the assigning code in the constructor
			//(from instantiation side condition)
			consPredBuffer.append(consAssignBuf);
			
			//now the predicates can be evaluated
			for ( int i = 0 ; i < predGroundAfterConst.size() ; i++ )
			{
				ExpressionElement predicate = (ExpressionElement) predGroundAfterConst.get(i);
				consPredBuffer.append( "if ( !(");
				consPredBuffer.append(generateExpressionCode(predicate));
				consPredBuffer.append(").getValue() )\n");
				consPredBuffer.append("{\n");
				consPredBuffer.append("//side condition predicate doesn't hold:\n");
				consPredBuffer.append("//step won't be instantiated\n");
				consPredBuffer.append("return false;\n");
				consPredBuffer.append("}\n");
			}
			
			consPredBuffer.append("} //end predicate-checking block\n");
		
		}
		obs.resetChanged();
		
		//final del mtodo constructorPrecondition, que es siempre igual
		consPredBuffer.append ( "return true;\n");
		consPredBuffer.append("} //try\n");
		consPredBuffer.append("catch ( ClassCastException cce )\n");
		consPredBuffer.append("{\n");
		consPredBuffer.append("return false;");
		consPredBuffer.append("} //catch\n");
		consPredBuffer.append ( "}\n" );
		
		methods.append(consPredBuffer);
		
		//esqueletos para el paso deductivo (para StepHandler):
		/*
		 * En estos momentos, hemos generado el cdigo del constructor, y nada ms.
		 * Eso quiere decir que, ahora mismo, tenemos marcados como "ground" en el
		 * paso deductivo aquellos elementos que, o bien son "ground" ya desde el
		 * principio (aparecen como constantes en el paso, no tienen atributo,
		 * cumplen elt.isGround()) o bien el constructor los convierte en "ground"
		 * (sern constantes en cada instancia del paso, tienen un atributo asociado
		 * pero que en estos momentos cumple this.isGround(this.attributeFor(elt));
		 * 
		 * Estos elementos son precisamente los que tienen que aparecer en los
		 * esqueletos asociados al paso deductivo (StepSkeletons). Nos definen
		 * los conjuntos de items a los que el paso es aplicable.
		 */
		//mtodo v()
		AntecedentNode antecedente = dsn.getAntecedent();
		int nItemsAntecedente = antecedente.numChildren();
		methods.append(
		"/**\n* Devuelve las especificaciones de bsqueda asociados a este paso deductivo. Estas especificaciones " +
		" definen los conjuntos de items a los que el paso es aplicable, es decir," +
		" son definiciones de conjuntos de items que se pueden ajustar a cada uno de los" +
		" items del antecedente.\n**/\n");
		methods.append("public List getAssociatedSpecifications ( )\n");
		methods.append("{\n");
		methods.append("List result = new ArrayList();\n");
		/*para cada item del antecedente*/
		for ( int i = 0 ; i < nItemsAntecedente ; i++ )
		{
			ItemNode anAntecItem = (ItemNode) antecedente.getChild(i);
			
			//construir generic wrapper element para el item del antecedente
			//(para visitarlo con el SkeletonVisitor).
			Element[] ar = new Element[anAntecItem.numChildren()];
			for ( int k = 0 ; k < anAntecItem.numChildren() ; k++ )
			{
				ElementNode en = (ElementNode) anAntecItem.getChild(k);
				ar[k] = en.getElement();
			}
			GenericWrapperElement gwe = new GenericWrapperElement(ar);
			
			//construir variable para el esqueleto asociado a este item del antecedente
			String skelName = VAR_PREFFIX + "mySkeleton" + "_" + String.valueOf(i);
			ApplySpecificationVisitor asv = new ApplySpecificationVisitor ( skelName , methods , this );
			gwe.accept(asv);
			stepSearchSkeletonTrees.add ( asv.getSearchDescriptor() );
			methods.append("result.add ( " + skelName + " );\n");
		}
		methods.append("return result;\n");
		methods.append("}\n");
		//ntese que, adems de generar el mtodo, se ha rellenado la lista
		//stepSearchSkeletonTrees.		
		
		//mtodo getInstancesForGrammar
		methods.append ( "public static List getInstancesForGrammar ( Grammar grammar , List toAddTo )\n" );
		methods.append ( "{\n" );
		if ( condiciones.numChildren() == 1 )
		{
			methods.append ( "for ( int i = 0 ; i < grammar.size() ; i++ )\n" );
			methods.append ( "{\n" );
			methods.append ( "Element[] regla = grammar.getEntity(i);\n" );
			methods.append ( "if ( constructorPrecondition(new Object[]{regla}) )\n" );
			methods.append ( "toAddTo.add( new " + className + "(new Object[]{regla}) );\n" );
			methods.append ( "}\n" );
		}
		else if ( condiciones.numChildren() == 0 )
		{
			methods.append("if ( constructorPrecondition(new Object[0]) )\n");
			methods.append("toAddTo.add( new " + className + "(new Object[0]) );\n");
		}
		else
		{
			System.err.println("[ERROR] Steps with more than one anchor are not supported by now.");
		}
		methods.append ( "return toAddTo;\n" );
		methods.append ( "}\n" );
		
		//mtodo hasEmptyAntecedent()
		/*
		 * Este mtodo permite saber si un paso deductivo tiene antecedente vaco o no.
		 * Los pasos con antecedente vaco sern ejecutados al principio y slo una vez.
		 */
		methods.append ( "public boolean hasEmptyAntecedent ()\n" );
		methods.append ( "{\n");
		if ( dsn.getAntecedent().numChildren() > 0 )
			methods.append ( "return false;\n" );
		else
			methods.append ( "return true;\n" );
		methods.append ( "}\n");
	
		
		//mtodo applyTo
		
		methods.append ( "public List applyTo ( Object[] item )\n" );
		methods.append ( "{\n" );
		methods.append ( "List result = null;\n" );
//		[2004-12-06.1] 		methods.append("resultIsValid = false;\n");
		
		if ( nItemsAntecedente > 0 )
		{
			//Llamar a los mtodos applyToi para aplicar el trigger item
			//empezando por cada item del antecedente.
			for ( int i = 0 ; i < nItemsAntecedente ; i++ )
				methods.append ( "result = applyTo" + i + " (item,result);\n");					
		}
		else
		{
			//Ningn item en el antecedente, simplemente devolver el consecuente.
			ItemNode consecuente = (ItemNode) dsn.getConsequent();
			
			 methods.append("if (called) return result; else called = true;\n");
			
			//construir generic wrapper
			Element[] conseqArray = new Element[consecuente.numChildren()];
			for ( int k = 0 ; k < consecuente.numChildren() ; k++ )
			{
				ElementNode en = (ElementNode) consecuente.getChild(k);
				conseqArray[k] = en.getElement();
			}
			GenericWrapperElement cWrapper = new GenericWrapperElement(conseqArray);
			
			CodeGeneratingVisitor cgv = new ApplySpecificationVisitor ( "resultItem" , methods , this );
			cWrapper.accept(cgv); //consequent wrapper accepts apply skel. visitor: generate result item skel.
			 methods.append("result = new ArrayList();\n");
			methods.append("result.add(resultItem);\n");
			
			/*
			 * Generacin del "existence skeleton tree". Este mismo cdigo se hace tambin para los items
			 * que tienen antecedente; pero est dentro de los bucles que generan el cdigo, pues se ha 
			 * de hacer una vez que todo lo que se hace "ground" durante la ejecucin del paso ya es "ground".
			 */			
			if ( existenceSkeletonTrees.size() < 1 )
			{
			
				ItemNode conseqItem = (ItemNode) dsn.getConsequent();
			
				//construir generic wrapper element para el item del consecuente
				//(para visitarlo con el SpecificationVisitor).
				Element[] arr = new Element[conseqItem.numChildren()];
				for ( int k = 0 ; k < conseqItem.numChildren() ; k++ )
				{
					ElementNode en = (ElementNode) conseqItem.getChild(k);
					arr[k] = en.getElement();
				}
				GenericWrapperElement aGwe = new GenericWrapperElement(arr);
				
				//construir variable para la especificacin de bsqueda asociada a este item del antecedente
				ApplySpecificationVisitor asv = new ApplySpecificationVisitor ( "" , new StringBuffer() , this );
				aGwe.accept(asv);
				existenceSkeletonTrees.add ( asv.getSearchDescriptor() );
			
			}
			
			
		}
		
		 methods.append ( "return result;\n" );
		methods.append ( "}\n" );
		
		//mtodos applyTo{i}
		
		for ( int i = 0 ; i < nItemsAntecedente ; i++ )
		{
			
			
			//mtodo applyTo{fixed i}			
			ItemNode itemToMatch = (ItemNode) antecedente.getChild(i);	
			
			//aqu hay que tener en cuenta cuales son ground: groundAttrList.contains(...)
			
			methods.append ( "private List applyTo" + i + " ( Object[] item , List result )\n" );
			methods.append ( "{\n" );
						
			//1. chequeo estructural (de longitudes).
						
			//construir generic wrapper
			Element[] ar = new Element[itemToMatch.numChildren()];
			for ( int j = 0 ; j < itemToMatch.numChildren() ; j++ )
			{
				ElementNode en = (ElementNode) itemToMatch.getChild(j);
				ar[j] = en.getElement();
			}
			GenericWrapperElement gwe = new GenericWrapperElement(ar);
			
			//generar el cdigo de chequeo estructural
			methods.append("//chequeo estructural\n");
			CodeGeneratingVisitor cgv = new ApplyInitialCheckVisitor ( "item" , methods , this );
			gwe.accept(cgv);
			
			//2. trigger item matching
			methods.append("//\"matching\" con el item disparador\n");
			ApplyTriggerMatchVisitor matchVisitor = new ApplyTriggerMatchVisitor ( "item" , methods , this );
			obs.recalculateState();
			obs.resetChanged();
			gwe.accept(matchVisitor);
			List changedPredicates = obs.getChangedPredicates();
			
			//cdigo de recuperacin, que deshace las asignaciones de variables.
			//Dentro de los visitadores, se utiliza para deshacer los cambios
			//que se hayan hecho si nos damos cuenta de que no hay matching.
			//Aqu (fuera de los visitantes) se utiliza ms tarde para aadirlo
			//al final del mtodo, porque aunque haya matching no querremos dejar
			//las asignaciones hechas en el paso deductivo (estaramos cambiando
			//el objeto paso)
			StringBuffer recoveryCode = matchVisitor.getRecoveryCodeBuffer();
			
			//check predicates grounded after trigger item matching
			for ( int w = 0 ; w < changedPredicates.size() ; w++ )
			{
				ExpressionElement predicate = (ExpressionElement) changedPredicates.get(w);
				methods.append( "if ( !(");
				methods.append(generateExpressionCode(predicate));
				methods.append(").getValue() )\n");
				methods.append("{\n");
				methods.append("//side condition predicate doesn't hold:\n");
				methods.append("//restore step state and return\n");
				methods.append(recoveryCode);
				methods.append("return result;\n");
				methods.append("}\n");
			}
			
			//3. foreach other item in antecedent, search for matching items
			//using ItemHandler
			
			//Cdigo de cierre de los bucles y dems. (para evitar recursividad,
			//usamos este buffer como una pila)
			StringBuffer appendAfterwards = new StringBuffer("");
			
			List toUndo = new ArrayList(); //of applyTriggerAssignVisitors, to undo changes
			
			for ( int j = 0 ; j < nItemsAntecedente ; j++ )
			{
				if ( j != i )
				{
					//look for matches for antecedent item j
					itemToMatch = (ItemNode) antecedente.getChild(j);	
					
					//construir generic wrapper
					ar = new Element[itemToMatch.numChildren()];
					for ( int k = 0 ; k < itemToMatch.numChildren() ; k++ )
					{
						ElementNode en = (ElementNode) itemToMatch.getChild(k);
						ar[k] = en.getElement();
					}
					gwe = new GenericWrapperElement(ar);
					
					//3.1. Build item search specification.
					String skelName = VAR_PREFFIX + "skeleton" + "_" + className + String.valueOf(i) + "_" + String.valueOf(j);
					ApplySpecificationVisitor asv = new ApplySpecificationVisitor ( skelName , methods , this );
					gwe.accept(asv);
					
					itemSearchSkeletonTrees.add ( asv.getSearchDescriptor() );
					
					//3.2. Pedir a la ItemHandler items de esta forma.
					methods.append("List items" + String.valueOf(j) + " = ItemHandler.instance().getBySpecification ( " + skelName + " );\n");
					
					methods.append("if ( items" + String.valueOf(j) + "!= null && items" + String.valueOf(j) + ".size() != 0 )\n" );
					methods.append("{\n");
					methods.append("for ( int ind" + String.valueOf(j) + " = 0 ; ind" + String.valueOf(j) + " < " + "items" + String.valueOf(j) + ".size() ; ind" + String.valueOf(j) + "++ )\n" );
					methods.append("{\n");
				}	
				//if caso base, procesar consecuente, aadir a lista de resultados.
				boolean baseCase; 
				//con ms de un item en antecedente, ver si este item es el ltimo
				baseCase = ( (j!=i) && (j == nItemsAntecedente - 1 || ( i == nItemsAntecedente-1 && j == nItemsAntecedente-2 )) );
				//si tenemos solo un item, estamos ya en el base.
				baseCase = baseCase || ( nItemsAntecedente == 1 );
				if ( baseCase ) 
				{
					//ltimo item antecedente: caso base.
					///pues j va hacia adelante saltandose i (siempre nos saltamos i salvo si hay un solo item).
					//3.2a.1: add resulting items to result
					ApplyNonTriggerAssignVisitor assignV = null;
					if ( nItemsAntecedente > 1 )
					{
						//ltimo item no-trigger
						assignV = new ApplyNonTriggerAssignVisitor ( "((Object[])items" + String.valueOf(j) + ".get(" + "ind" + String.valueOf(j) + "))" , methods , this  );
						obs.resetChanged();
						gwe.accept(assignV);
						List changedPredicatesHere = obs.getChangedPredicates();
						//check predicates grounded after nontrigger item matching
						for ( int w = 0 ; w < changedPredicatesHere.size() ; w++ )
						{
							ExpressionElement predicate = (ExpressionElement) changedPredicatesHere.get(w);
							methods.append( "if ( !(");
							methods.append(generateExpressionCode(predicate));
							methods.append(").getValue() )\n");
							methods.append("{\n");
							methods.append("//side condition predicate doesn't hold:\n");
							methods.append("//restore step state and continue\n");
							methods.append(assignV.getRecoveryCodeBuffer());
							methods.append("continue;\n");
							methods.append("}\n");
						}
					}
					
					
	
					
					//[added 2006-11-06]
					//construimos el skeleton para los items negados, que van a ser ground (si el
					//paso deductivo esta bien formado) con lo cual lo que estamos haciendo es
					//construir un item (igual que en el consecuente, ver mas abajo). Este item
					//se chequea con el exists.
					for ( int w = 0 ; w < dsn.getNegations().numChildren(); w++ )
					{
						methods.append("//negation check " + (w+1) + "\n");
						ItemNode negatedItem = (ItemNode)dsn.getNegations().getChild(w);
						
						//construir generic wrapper
						Element[] negArray = new Element[negatedItem.numChildren()];
						for ( int k = 0 ; k < negatedItem.numChildren() ; k++ )
						{
							ElementNode en = (ElementNode) negatedItem.getChild(k);
							negArray[k] = en.getElement();
						}
						GenericWrapperElement cWrapper = new GenericWrapperElement(negArray);
						
						String varName = "negatedItem_"+w;
						
						cgv = new ApplySpecificationVisitor ( varName , methods , this );
						cWrapper.accept(cgv); //negated wrapper accepts apply skel. visitor: generate negated item skel.
							
						methods.append( "if ( ItemHandler.instance().exists( " + varName + " ) )" );
						methods.append("{\n");
						methods.append("//\"not\" condition doesn't hold:\n");
						//methods.append("//don't continue, it obviously won't hold with other instantiations either\n");
						//methods.append(assignV.getRecoveryCodeBuffer());
						//methods.append(" ArrayList();\n");
						
						if ( nItemsAntecedente > 1 ) 
						{
							methods.append("//restore step state and continue\n");
							methods.append(assignV.getRecoveryCodeBuffer());
							methods.append("continue;\n");
							methods.append("}\n");
						}
						else
						{
							methods.append("//return\n");
							methods.append("return result;\n");
							methods.append("}\n");
						}

						
						
					}
					
					
					//construimos el skeleton para el item consecuente.
					//el skeleton va a ser ground (si el paso deductivo esta bien formado)
					//con lo cual lo que estamos haciendo es construir un item, que sera
					//el resultado.
					
					ItemNode consecuente = (ItemNode) dsn.getConsequent();
					
					//construir generic wrapper
					Element[] conseqArray = new Element[consecuente.numChildren()];
					for ( int k = 0 ; k < consecuente.numChildren() ; k++ )
					{
						ElementNode en = (ElementNode) consecuente.getChild(k);
						conseqArray[k] = en.getElement();
					}
					GenericWrapperElement cWrapper = new GenericWrapperElement(conseqArray);
					
					cgv = new ApplySpecificationVisitor ( "resultItem" , methods , this );
					cWrapper.accept(cgv); //consequent wrapper accepts apply skel. visitor: generate result item skel.
								
					methods.append("if ( result == null ) result = new ArrayList();\n");
					
					methods.append("result.add(resultItem);\n");					  
					
					
					/*
					 * Aqu lo tenemos todo en el estado en que est cuando se devuelve
					 * un resultado. Es el momento de crear el esqueleto asociado a este
					 * paso en el "existence index", si es que no se ha creado ya.
					 * (da igual en qu camino de ejecucin del paso deductivo lo creemos,
					 * pues el item resultante va a ser de la misma forma en cualquiera de
					 * ellos, as que simplemente lo creamos la primera vez que pasemos por
					 * esta parte del cdigo)
					 */
					if ( existenceSkeletonTrees.size() < 1 )
					{
					
						ItemNode conseqItem = (ItemNode) dsn.getConsequent();
					
						//construir generic wrapper element para el item del consecuente
						//(para visitarlo con el SkeletonVisitor).
						Element[] arr = new Element[conseqItem.numChildren()];
						for ( int k = 0 ; k < conseqItem.numChildren() ; k++ )
						{
							ElementNode en = (ElementNode) conseqItem.getChild(k);
							arr[k] = en.getElement();
						}
						GenericWrapperElement aGwe = new GenericWrapperElement(arr);
						
						//construir variable para el esqueleto asociado a este item del antecedente
						ApplySpecificationVisitor asv = new ApplySpecificationVisitor ( "" , new StringBuffer() , this );
						aGwe.accept(asv);
						existenceSkeletonTrees.add ( asv.getSearchDescriptor() );
					
					}
					
					if ( assignV != null )
					{
						assignV.undoChanges();
						obs.recalculateState();
						obs.resetChanged();					
						methods.append(assignV.getRecoveryCodeBuffer());
					}
					
				}
				if ( j != i )
				{				
					appendAfterwards.insert(0,"} //end if there are items at items" + j +"\n");
					appendAfterwards.insert(0,"} //end foreach item at items" + j + "\n");	
				}
				if ( !baseCase && j!=i )
				{
					//if not ltimo item, asignar valores y continuar iterando
					//por resto de items de antecedente.
					
					//not base case (not the last item)
					
					//3.2a.1: add resulting items to result
					ApplyNonTriggerAssignVisitor assignV = new ApplyNonTriggerAssignVisitor ( "((Object[])(items" + String.valueOf(j) + ".get(" + "ind" + String.valueOf(j) + ")))" , methods , this  );
					
					obs.resetChanged();
					gwe.accept(assignV);
					List changedPredicatesHere = obs.getChangedPredicates();
					//check predicates grounded after nontrigger item matching
					for ( int w = 0 ; w < changedPredicatesHere.size() ; w++ )
					{
						ExpressionElement predicate = (ExpressionElement) changedPredicatesHere.get(w);
						methods.append( "if ( !(");
						methods.append(generateExpressionCode(predicate));
						methods.append(").getValue() )\n");
						methods.append("{\n");
						methods.append("//side condition predicate doesn't hold:\n");
						methods.append("//restore step state and continue\n");
						methods.append(assignV.getRecoveryCodeBuffer());
						methods.append("continue;\n");
						methods.append("}\n");
					}
					
					toUndo.add(assignV);
					
					appendAfterwards.insert(0,assignV.getRecoveryCodeBuffer());
					//antes de cerrar el foreach
					
				} //elif
				
			}
			
			
			methods.append(appendAfterwards);
			
			methods.append("//dejar el estado del objeto paso como estaba\n");
			methods.append(recoveryCode); //al final deshacemos cambios y devolvemos result
			methods.append("return result;\n");
			
			methods.append ( "}\n" );
					
			
			matchVisitor.undoChanges(); //vuelve a no-ground los atributos matcheados
			
			for ( int j = 0 ; j < toUndo.size() ; j++ )
			{
				ApplyNonTriggerAssignVisitor atav = (ApplyNonTriggerAssignVisitor) toUndo.get(j);
				atav.undoChanges();
				obs.recalculateState();
				obs.resetChanged();
			}
			
		}
		
		//return skeletonTrees;
		
	}
	
	
	/**
	 * Nos da los nodos hoja de un rbol sintctico de raz el ParseTreeNode ptn.
	 * @param ptn Nodo raz del rbol.
	 * @param leafNodes Lista donde quedarn los nodos hoja. Debera estar inicializada
	 * a vaco al llamar este mtodo.
	 */
	private void getLeafNodes ( ParseTreeNode ptn , List leafNodes )
	{
		//preorden.
		if ( ptn.numChildren() == 0 )
		{
			leafNodes.add(ptn);
		}
		else
		{
			for ( int i = 0 ; i < ptn.numChildren() ; i++ )
			{
				ParseTreeNode son = ptn.getChild(i);
				getLeafNodes(son,leafNodes);
			}
		}
	}
	
	/**
	 * Genera los atributos, que siempre se van a corresponder con los elementos
	 * raz (elementos no composite).
	 * Como efecto lateral, las declaraciones de los atributos quedan en el string
	 * buffer attributes.
	 * Como efecto lateral, los elementos del rbol a los que se refieren los
	 * atributos quedan guardados en la lista privada attrElementList. As en el anlisis
	 * posterior se sabr qu atributo se corresponde con cada elemento del rbol. 
	 * Asimismo, los atributos quedan en attrList.
	 */
	private void generateAttributes ( )
	{
		
		//los atributos se correspondern con los nodos hoja que haya en el antecedente, el
		//consecuente y los predicados.
		
		List leafNodes1 = new ArrayList();
		List leafNodes2 = new ArrayList();
		List leafNodes3 = new ArrayList();
		
		getLeafNodes ( dsn.getAntecedent() , leafNodes1 );
		getLeafNodes ( dsn.getConsequent() , leafNodes2 );
		getLeafNodes ( dsn.getPredicates() , leafNodes3 );
		
		//recorrer nodos hoja del antecedente.
		//normalmente sern nodos correspondientes a elementos (ElementNode),
		//salvo en un caso que se ver.
		for ( int i = 0 ; i < leafNodes1.size() ; i++ )
		{
			ElementNode en;
			try
			{
				en = (ElementNode) leafNodes1.get(i);
			}
			catch ( ClassCastException cce )
			{
				/*
				 * Cundo se entra en este catch?
				 * Cuando tenemos un antecedente sin items, pues en este
				 * caso el nodo antecedente ser interpretado como "leaf node",
				 * no habiendo nodos correspondientes a elementos dentro
				 * de l.
				 */
				//Este tipo de pasos se deben hacer slo una vez, llevamos
				//un flag para ver si ya se han hecho.
				attributes.append("boolean called = false;\n");
				break;
			}
			Element elt = en.getElement();
			List leafElements = elt.getLeafElements();
			for ( int j = 0 ; j < leafElements.size() ; j++ )
			{
				Element leafElt = (Element) leafElements.get(j);
				if ( !attrElementList.contains(leafElt) && !leafElt.isGround() )
				{			
					int dimension = (leafElt instanceof SequenceElement)?1:0;
					Attribute attr = new Attribute ( leafElt , dimension );
					attributes.append("\t");
					attributes.append( attr.getDeclaration() );
					attributes.append("\n");
					attrElementList.add(leafElt);
					attrList.add(attr);
				
				}
			}
		}
		for ( int i = 0 ; i < leafNodes2.size() ; i++ )
		{
			ElementNode en;
			try
			{
				en = (ElementNode) leafNodes2.get(i);
			}
			catch ( ClassCastException cce )
			{
				//consecuente vaco, seguramente porque sea un paso meta.
				//Por ello se nos devuelve como nodo hoja el ItemNode en lugar
				//de un ElementNode.
				//No hacemos nada.
				break;
			}
			Element elt = en.getElement();
			List leafElements = elt.getLeafElements();
			for ( int j = 0 ; j < leafElements.size() ; j++ )
			{		
				Element leafElt = (Element) leafElements.get(j);
				if ( !attrElementList.contains(leafElt ) && !leafElt.isGround() )
				{
					int dimension = (leafElt instanceof SequenceElement)?1:0;
					Attribute attr = new Attribute ( leafElt , dimension );
					attributes.append("\t");
					attributes.append( attr.getDeclaration() );
					attributes.append("\n");
					attrElementList.add(leafElt);
					attrList.add(attr);
				}
			}
		}
		for ( int i = 0 ; i < leafNodes3.size() ; i++ )
		{
			ElementNode en;
			try
			{
				en = (ElementNode) leafNodes3.get(i);
			}
			catch ( ClassCastException cce )
			{
				//Aqu entramos si no hay predicados (el nodo de predicados est vaco).
				//No es problema, en este caso no hacemos nada.
				break;
			}
			Element elt = en.getElement();
			List leafElements = elt.getLeafElements();
			for ( int j = 0 ; j < leafElements.size() ; j++ )
			{		
				Element leafElt = (Element) leafElements.get(j);
				if ( !attrElementList.contains(leafElt ) && !leafElt.isGround() )
				{
					int dimension = (leafElt instanceof SequenceElement)?1:0;
					Attribute attr = new Attribute ( leafElt , dimension );
					attributes.append("\t");
					attributes.append( attr.getDeclaration() );
					attributes.append("\n");
					attrElementList.add(leafElt);
					attrList.add(attr);
				}
			}
		}
		
	}


	/**
	 * Genera el cdigo de la clase correspondiente al paso deductivo descrito por el nodo
	 * dado. Adems, rellena dos listas de descriptores de bsqueda: itemSearchSkelTrees y
	 * stepSearchSkelTrees. En la primera pone la informacin sobre qu tipo de bsquedas
	 * van a utilizar los mtodos de este paso para buscar items, y en la segunda pone la
	 * informacin sobre qu tipo de bsquedas se van a usar en el programa para buscar
	 * este paso a partir de un item.
	 * @param dsn Nodo del paso deductivo para el que queremos generar el cdigo.
	 * @param itemSearchSkelTrees Lista de descriptores de bsqueda a la que
	 * este mtodo aade los descriptores de las especificaciones
	 * que se utilizarn en este paso deductivo para buscar items.
	 * @param stepSearchSkelTrees Lista a la que este mtodo aadir
	 * los descriptores de bsqueda asociados a este
	 * paso deductivo, que es utilizarn para buscar el paso a aplicar.
	 * @param existenceSkelTrees Lista de descriptores de bsqueda a la que este mtodo aade
	 * los esqueletos que se utilizarn para determinar la existencia en la ItemHandler
	 * (mtodo exists) de los items de la forma del consecuente de este paso (i.e. los items
	 * producidos por este paso)
	 * @return Cdigo de la clase que representa el paso deductivo.
	 */
		public String generateCode (  List itemSearchSkelTrees , List stepSearchSkelTrees , List existenceSkelTrees )
		{
			//generate code for one deductive step and put skeleton descriptor
			//trees in given list
			
			attributes = new StringBuffer("");
			methods = new StringBuffer("");
			attrElementList = new ArrayList();
			attrList = new ArrayList();
			groundAttrList = new ArrayList();
			
			StringBuffer stepSb = new StringBuffer("");
			
			if ( dsn.isGoalStep() )
				className = CLASS_PREFFIX + "Goal" + (++goalsGenerated);
			else	
				className = CLASS_PREFFIX + dsn.getName() + "Step";
			
			stepSb.append("/*Code Generated for File: " + className + ".java*/\n");
			
			stepSb.append("\n");
			
			stepSb.append("package schema.step;\n");
			stepSb.append("import element.*;\n");
			stepSb.append("import schema.item.*;\n");
			stepSb.append("import java.util.*;\n");
			stepSb.append("import grammar.Grammar;\n");
			
			stepSb.append("\n");
			
			stepSb.append("public class " + className + " implements DeductiveStep\n");
			
			stepSb.append("{\n\n");
			
			generateAttributes();
			
			stepSb.append("\t//Attributes:\n\n");
			
			stepSb.append(attributes.toString());
			
			stepSb.append("\n\n");
			
			//generate method code and fill skeleton trees list
			List theItemSearchSkelTrees = new ArrayList();
			List theStepSearchSkelTrees = new ArrayList();
			List theExistenceSkelTrees = new ArrayList();
			generateMethods(theItemSearchSkelTrees,theStepSearchSkelTrees,theExistenceSkelTrees);
			
			stepSb.append("\t//Methods:\n\n");
			
			stepSb.append(methods.toString());
			
			stepSb.append("}\n");
			
			itemSearchSkelTrees.addAll(theItemSearchSkelTrees);
			stepSearchSkelTrees.addAll(theStepSearchSkelTrees);
			existenceSkelTrees.addAll(theExistenceSkelTrees);
			
			return new String(stepSb);
			
		}
	
}
