package grammar.tag.util;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

import org.w3c.dom.Element;

import util.DOMNodeListWrapper;
import util.DOMNodeWrapper;

public class HeadCornerInformationAdder extends AbstractGrammarTransformer
{

	private Comparator comp = new HeadCornerDefaultComparator();
	
	public String toString()
	{
		return "Adding head-corner information";
	}
	
	public HeadCornerInformationAdder(String inFile, String outFile, AbstractGrammarTransformer next) 
	{
		super(inFile, outFile, next);
	}

	public HeadCornerInformationAdder(String inFile, String outFile) 
	{
		super(inFile, outFile);
	}

	public HeadCornerInformationAdder(AbstractGrammarTransformer next) 
	{
		super(next);
	}
	
	public void addHeadInfoToNode ( DOMNodeWrapper node , int curIndex )
	{
		((Element)node.getNode()).setAttribute("headind",String.valueOf(curIndex));
		DOMNodeListWrapper children = node.getDirectChildrenElementsByTagName("node");
		List sortable = new ArrayList();
		System.out.println("Parent node: " + node.getNode());
		System.out.println("Parent tag: " + ((Element)node.getNode()).getTagName());
		System.out.println("Parent cat: " + ((Element)node.getNode()).getAttribute("cat"));
		for ( int i = 0 ; i < children.size() ; i++ )
		{
			System.out.println("A child node: " + children.get(i).getNode());
			System.out.println("A child tag: " + ((Element)children.get(i).getNode()).getTagName());
			System.out.println("A child cat: " + ((Element)children.get(i).getNode()).getAttribute("cat"));
			sortable.add(children.get(i));
		}
		java.util.Collections.sort(sortable,comp);
		for ( int i = 0 ; i < sortable.size() ; i++ )
		{
			addHeadInfoToNode( (DOMNodeWrapper)sortable.get(i) , curIndex+i+1 );
		}
	}
	
	public void preprocessTreeNode(DOMNodeWrapper treeNode)
	{
		DOMNodeListWrapper rootNodes = treeNode.getDirectChildrenElementsByTagName("node");
		for ( int i = 0 ; i < rootNodes.size() ; i++ )
		{
			addHeadInfoToNode ( rootNodes.get(i) , 0 );
		}
	}
	
}
