/*
 * Created on 02-ago-2004
 *
 */
package parser.eparser;
import java.lang.reflect.*;

/**
 * CreationMethodAdapter es un adaptador para dar un interfaz uniforme a constructores
 * (clase Constructor) y mtodos (clase Method) a la hora de crear objetos.
 * @author carlos
 *
 */
public class CreationMethodAdapter 
{
	
	Constructor c = null;
	Method m = null;
	
	/**
	 * Crea un "wrapper" para un constructor.
	 * @param c Constructor que estamos adaptando.
	 */
	public CreationMethodAdapter ( Constructor c )
	{
		this.c = c;
	}
	
	/**
	 * Crea un "wrapper" para un mtodo.
	 * @param m Mtodo que estamos adaptando.
	 */
	public CreationMethodAdapter ( Method m )
	{
		this.m = m;
	}
	
	/**
	 * Invoca el constructor o el mtodo con los argumentos dados.
	 * @param args Argumentos para invocar el constructor o mtodo.
	 * @return La instancia de objeto que se ha creado (o que devuelve el mtodo).
	 * @throws IllegalAccessException Si no tenemos acceso a la definicin de lo que queremos crear.
	 * @throws InstantiationException Si lo que queremos instanciar es un interfaz o clase abstracta.
	 * @throws InvocationTargetException Si el mtodo o constructor que estamos invocando tira una excepcin. Dicha excepcin se obtiene con getCause().
	 */
	public Object invoke ( Object[] args ) throws
												IllegalAccessException,
												InstantiationException,
												InvocationTargetException
	{
		if ( m != null )
		{
			return m.invoke ( null , args );
		}
		else
		{
			return c.newInstance ( args );
		}
	}
	
}
