package util;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.StringTokenizer;

/**
 * This class implements an extremely simple part-of-speech tagger.
 * 
 * The tagger receives a file containing sentences composed of terminals, and another file with terminal-to-preterminal associations,
 * and writes to a third file the corresponding sentences of preterminals.
 * 
 * @author Carlos Gmez
 * Created at mortadelo, 28/11/2006 16:16:56
 *
 */
public class SentencesToTags 
{
	
	public static String IN_FILE = "C:\\Proyecto\\Version9\\grammars\\atis-sentences.txt";
	public static String OUT_FILE = "C:\\Proyecto\\Version9\\grammars\\atis-sentences-tagged.txt";
	public static String LEX_FILE = "C:\\Proyecto\\Version9\\grammars\\atis-lex.txt";
	
	public static void main ( String[] args ) throws FileNotFoundException,IOException
	{
	
		tag ( new File(IN_FILE) , new File(LEX_FILE) , new File(OUT_FILE) );
		
	}
	
	public static void tag ( File input , File lexicon , File output ) throws FileNotFoundException,IOException
	{
	
		BufferedReader br = new BufferedReader ( new InputStreamReader ( new FileInputStream ( input ) ) );
		BufferedReader brLex = new BufferedReader ( new InputStreamReader ( new FileInputStream ( lexicon ) ) );
		PrintWriter pw = new PrintWriter ( new OutputStreamWriter ( new FileOutputStream ( output ) ) );
		
		//read lexicon into hashtable
		HashMap lexMap = new HashMap();
		String lexLine;
		
		while ( (lexLine = brLex.readLine()) != null )
		{
			if ( lexLine.startsWith(";") || lexLine.trim().length() == 0 ) ;
			else
			{
				StringTokenizer st = new StringTokenizer(lexLine);
				String word = st.nextToken().trim();
				String tag = st.nextToken().trim();
				lexMap.put(word,tag);
			}
		}
		
		//read sentences and tag'em
		String sentLine;
		
		while ( (sentLine = br.readLine() ) != null )
		{
			if ( sentLine.startsWith(";") || sentLine.trim().length() == 0 ) ;
			else
			{
				StringTokenizer st = new StringTokenizer ( sentLine );
				StringBuffer lineToWrite = new StringBuffer("");
				while ( st.hasMoreTokens() )
				{
					String word = st.nextToken().trim();
					String tag = (String)lexMap.get(word);
					if ( tag == null )
						tag = word; //words without an associated tag are left as they were
					lineToWrite.append(" ");
					lineToWrite.append(tag);
				}
				pw.println(lineToWrite.toString().substring(1));
			}
		}
		
		pw.close();
		
	}
	

}
